@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2018 Insecure.Com LLC ("The Nmap  *
+ * The Nmap Security Scanner is (C) 1996-2019 Insecure.Com LLC ("The Nmap  *
  * Project"). Nmap is also a registered trademark of the Nmap Project.     *
  * This program is free software; you may redistribute and/or modify it    *
  * under the terms of the GNU General Public License as published by the   *
@@ -128,7 +128,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOutputTable.cc 37126 2018-01-28 21:18:17Z fyodor $ */
+/* $Id: NmapOutputTable.cc 37640 2019-05-28 21:36:04Z dmiller $ */
 
 #ifdef WIN32
 #include "nmap_winconfig.h"
@@ -233,7 +233,7 @@ void NmapOutputTable::addItemFormatted(unsigned int row,
   res = Vsnprintf(buf, sizeof(buf), fmt, ap);
   va_end(ap);
 
-  if (res > sizeof(buf))
+  if (res >= sizeof(buf))
     fatal("NmapOutputTable only supports adding up to 4096 to a cell via %s.", __func__);
 
   addItem(row, column, fullrow, true, buf, res);
