@@ -6,96 +6,136 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * The Nmap Security Scanner is (C) 1996-2019 Insecure.Com LLC ("The Nmap  *
+ * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+ * This program is free software; you may redistribute and/or modify it    *
+ * under the terms of the GNU General Public License as published by the   *
+ * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+ * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+ * right to use, modify, and redistribute this software under certain      *
+ * conditions.  If you wish to embed Nmap technology into proprietary      *
+ * software, we sell alternative licenses (contact sales@nmap.com).        *
+ * Dozens of software vendors already license Nmap technology such as      *
+ * host discovery, port scanning, OS detection, version detection, and     *
+ * the Nmap Scripting Engine.                                              *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * Note that the GPL places important restrictions on "derivative works",  *
+ * yet it does not provide a detailed definition of that term.  To avoid   *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
+ * derivative work for the purpose of this license if it does any of the   *
+ * following with any software or content covered by this license          *
+ * ("Covered Software"):                                                   *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * o Integrates source code from Covered Software.                         *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+ * or nmap-service-probes.                                                 *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * o Is designed specifically to execute Covered Software and parse the    *
+ * results (as opposed to typical shell or execution-menu apps, which will *
+ * execute anything you tell them to).                                     *
+ *                                                                         *
+ * o Includes Covered Software in a proprietary executable installer.  The *
+ * installers produced by InstallShield are an example of this.  Including *
+ * Nmap with other software in compressed or archival form does not        *
+ * trigger this provision, provided appropriate open source decompression  *
+ * or de-archiving software is widely available for no charge.  For the    *
+ * purposes of this license, an installer is considered to include Covered *
+ * Software even if it actually retrieves a copy of Covered Software from  *
+ * another source during runtime (such as by downloading it from the       *
+ * Internet).                                                              *
+ *                                                                         *
+ * o Links (statically or dynamically) to a library which does any of the  *
+ * above.                                                                  *
+ *                                                                         *
+ * o Executes a helper program, module, or script to do any of the above.  *
+ *                                                                         *
+ * This list is not exclusive, but is meant to clarify our interpretation  *
+ * of derived works with some common examples.  Other people may interpret *
+ * the plain GPL differently, so we consider this a special exception to   *
+ * the GPL that we apply to Covered Software.  Works which meet any of     *
+ * these conditions must conform to all of the terms of this license,      *
+ * particularly including the GPL Section 3 requirements of providing      *
+ * source code and allowing free redistribution of the work as a whole.    *
+ *                                                                         *
+ * As another special exception to the GPL terms, the Nmap Project grants  *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
  * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
+ * linked combinations including the two.                                  *
  *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
+ * The Nmap Project has permission to redistribute Npcap, a packet         *
+ * capturing driver and library for the Microsoft Windows platform.        *
+ * Npcap is a separate work with it's own license rather than this Nmap    *
+ * license.  Since the Npcap license does not permit redistribution        *
+ * without special permission, our Nmap Windows binary packages which      *
+ * contain Npcap may not be redistributed without special permission.      *
+ *                                                                         *
+ * Any redistribution of Covered Software, including any derived works,    *
+ * must obey and carry forward all of the terms of this license, including *
+ * obeying all GPL rules and restrictions.  For example, source code of    *
+ * the whole work must be provided and free redistribution must be         *
+ * allowed.  All GPL references to "this License", are to be treated as    *
+ * including the terms and conditions of this license text as well.        *
+ *                                                                         *
+ * Because this license imposes special exceptions to the GPL, Covered     *
+ * Work may not be combined (even as part of a larger work) with plain GPL *
+ * software.  The terms, conditions, and exceptions of this license must   *
+ * be included as well.  This license is incompatible with some other open *
+ * source licenses as well.  In some cases we can relicense portions of    *
+ * Nmap or grant special permissions to use it in other open source        *
+ * software.  Please contact fyodor@nmap.org with any such requests.       *
+ * Similarly, we don't incorporate incompatible open source software into  *
+ * Covered Software without special permission from the copyright holders. *
+ *                                                                         *
+ * If you have any questions about the licensing restrictions on using     *
+ * Nmap in other works, we are happy to help.  As mentioned above, we also *
+ * offer an alternative license to integrate Nmap into proprietary         *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing support and updates.  They also fund the continued         *
+ * development of Nmap.  Please email sales@nmap.com for further           *
+ * information.                                                            *
+ *                                                                         *
+ * If you have received a written license agreement or contract for        *
+ * Covered Software stating terms other than these, you may choose to use  *
+ * and redistribute Covered Software under those terms instead of these.   *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify            *
+ * otherwise) that you are offering the Nmap Project the unlimited,        *
  * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * will always be available Open Source, but this is important because     *
+ * the inability to relicense code has caused devastating problems for     *
+ * other Free Software projects (such as KDE and NASM).  We also           *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+ * license file for more details (it's in a COPYING file included with     *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: Target.h 21904 2011-01-21 00:04:16Z fyodor $ */
+/* $Id: Target.h 37640 2019-05-28 21:36:04Z dmiller $ */
 
 #ifndef TARGET_H
 #define TARGET_H
 
-#include "nmap.h"
-#include "FingerPrintResults.h"
+#include "nbase.h"
+
 #include "libnetutil/netutil.h"
 
 #ifndef NOLUA
@@ -104,28 +144,21 @@
 
 #include "portreasons.h"
 #include "portlist.h"
-#include "tcpip.h"
-#include "scan_engine.h"
+#include "probespec.h"
+#include "osscan.h"
+#include "osscan2.h"
+class FingerPrintResults;
 
 #include <list>
 #include <string>
+#include <time.h> /* time_t */
 
 #ifndef INET6_ADDRSTRLEN
 #define INET6_ADDRSTRLEN 46
 #endif
 
 enum osscan_flags {
-	OS_NOTPERF=0, OS_PERF, OS_PERF_UNREL
-};
-
-/* The method used to calculate the Target::distance, included in OS
-   fingerprints. */
-enum dist_calc_method {
-	DIST_METHOD_NONE,
-	DIST_METHOD_LOCALHOST,
-	DIST_METHOD_DIRECT,
-	DIST_METHOD_ICMP,
-	DIST_METHOD_TRACEROUTE
+        OS_NOTPERF=0, OS_PERF, OS_PERF_UNREL
 };
 
 struct host_timeout_nfo {
@@ -152,54 +185,62 @@ struct TracerouteHop {
 };
 
 class Target {
- public: /* For now ... a lot of the data members should be made private */
+ public: /* For now ... TODO: a lot of the data members should be made private */
   Target();
   ~Target();
   /* Recycles the object by freeing internal objects and reinitializing
      to default state */
   void Recycle();
+  /* Returns the address family of the destination address. */
+  int af() const;
   /* Fills a sockaddr_storage with the AF_INET or AF_INET6 address
      information of the target.  This is a preferred way to get the
      address since it is portable for IPv6 hosts.  Returns 0 for
      success. ss_len must be provided.  It is not examined, but is set
      to the size of the sockaddr copied in. */
-  int TargetSockAddr(struct sockaddr_storage *ss, size_t *ss_len);
+  int TargetSockAddr(struct sockaddr_storage *ss, size_t *ss_len) const;
+  const struct sockaddr_storage *TargetSockAddr() const;
   /* Note that it is OK to pass in a sockaddr_in or sockaddr_in6 casted
      to sockaddr_storage */
-  void setTargetSockAddr(struct sockaddr_storage *ss, size_t ss_len);
+  void setTargetSockAddr(const struct sockaddr_storage *ss, size_t ss_len);
   // Returns IPv4 target host address or {0} if unavailable.
   struct in_addr v4host() const;
   const struct in_addr *v4hostip() const;
+  const struct in6_addr *v6hostip() const;
   /* The source address used to reach the target */
-  int SourceSockAddr(struct sockaddr_storage *ss, size_t *ss_len);
+  int SourceSockAddr(struct sockaddr_storage *ss, size_t *ss_len) const;
+  const struct sockaddr_storage *SourceSockAddr() const;
   /* Note that it is OK to pass in a sockaddr_in or sockaddr_in6 casted
      to sockaddr_storage */
-  void setSourceSockAddr(struct sockaddr_storage *ss, size_t ss_len);
-  struct in_addr v4source() const;
+  void setSourceSockAddr(const struct sockaddr_storage *ss, size_t ss_len);
+  struct sockaddr_storage source() const;
   const struct in_addr *v4sourceip() const;
+  const struct in6_addr *v6sourceip() const;
   /* The IPv4 or IPv6 literal string for the target host */
   const char *targetipstr() const { return targetipstring; }
+  /* The IPv4 or IPv6 literal string for the source address */
+  const char *sourceipstr() const { return sourceipstring; }
   /* Give the name from the last setHostName() call, which should be
    the name obtained from reverse-resolution (PTR query) of the IP (v4
    or v6).  If the name has not been set, or was set to NULL, an empty
    string ("") is returned to make printing easier. */
-  const char *HostName() { return hostname? hostname : "";  }
-  /* You can set to NULL to erase a name or if it failed to resolve -- or 
+  const char *HostName() const { return hostname? hostname : "";  }
+  /* You can set to NULL to erase a name or if it failed to resolve -- or
      just don't call this if it fails to resolve.  The hostname is blown
      away when you setTargetSockAddr(), so make sure you do these in proper
      order
   */
-  void setHostName(char *name);
+  void setHostName(const char *name);
   /* Generates a printable string consisting of the host's IP
      address and hostname (if available).  Eg "www.insecure.org
      (64.71.184.53)" or "fe80::202:e3ff:fe14:1102".  The name is
      written into the buffer provided, which is also returned.  Results
      that do not fit in buflen will be truncated. */
-  const char *NameIP(char *buf, size_t buflen);
+  const char *NameIP(char *buf, size_t buflen) const;
   /* This next version returns a STATIC buffer -- so no concurrency */
-  const char *NameIP();
+  const char *NameIP() const;
 
-  /* Give the name from the last setTargetName() call, which is the 
+  /* Give the name from the last setTargetName() call, which is the
    name of the target given on the command line if it's a named
    host. */
   const char *TargetName() { return targetname; }
@@ -228,7 +269,7 @@ class Target {
   void setMTU(int devmtu);
   int MTU(void);
 
-  /* Sets the interface type to one of: 
+  /* Sets the interface type to one of:
      devt_ethernet, devt_loopback, devt_p2p, devt_other
    */
   void setIfType(devtype iftype) { interface_type = iftype; }
@@ -283,15 +324,14 @@ class Target {
   unsigned int flags; /* HOST_UNKNOWN, HOST_UP, or HOST_DOWN. */
   struct timeout_info to;
   char *hostname; // Null if unable to resolve or unset
-  char * targetname; // The name of the target host given on the commmand line if it is a named host
+  char * targetname; // The name of the target host given on the command line if it is a named host
 
   struct probespec traceroute_probespec;
   std::list <TracerouteHop> traceroute_hops;
 
   /* If the address for this target came from a DNS lookup, the list of
-     resultant addresses (sometimes there are more than one). The address
-     actually used is always the first element in this list. */
-  std::list<struct sockaddr_storage> resolved_addrs;
+     resultant addresses (sometimes there are more than one) that were not scanned. */
+  std::list<struct sockaddr_storage> unscanned_addrs;
 
 #ifndef NOLUA
   ScriptResults scriptResults;
@@ -309,14 +349,17 @@ class Target {
   private:
   void Initialize();
   void FreeInternal(); // Free memory allocated inside this object
- // Creates a "presentation" formatted string out of the IPv4/IPv6 address
-  void GenerateIPString();
+ // Creates a "presentation" formatted string out of the target's IPv4/IPv6 address
+  void GenerateTargetIPString();
+ // Creates a "presentation" formatted string out of the source IPv4/IPv6 address.
+  void GenerateSourceIPString();
   struct sockaddr_storage targetsock, sourcesock, nexthopsock;
   size_t targetsocklen, sourcesocklen, nexthopsocklen;
   int directly_connected; // -1 = unset; 0 = no; 1 = yes
   char targetipstring[INET6_ADDRSTRLEN];
-  char *nameIPBuf; /* for the NameIP(void) function to return */
-  u8 MACaddress[6], SrcMACaddress[6], NextHopMACaddress[6];  
+  char sourceipstring[INET6_ADDRSTRLEN];
+  mutable char *nameIPBuf; /* for the NameIP(void) function to return */
+  u8 MACaddress[6], SrcMACaddress[6], NextHopMACaddress[6];
   bool MACaddress_set, SrcMACaddress_set, NextHopMACaddress_set;
   struct host_timeout_nfo htn;
   devtype interface_type;
@@ -324,9 +367,10 @@ class Target {
   char devfullname[32];
   int mtu;
   /* 0 (OS_NOTPERF) if os detection not performed
-   * 1 (OS_PERF) if os detection performed 
+   * 1 (OS_PERF) if os detection performed
    * 2 (OS_PERF_UNREL) if an unreliable os detection has been performed */
-  int osscan_flag; 
+  int osscan_flag;
 };
 
 #endif /* TARGET_H */
+
