@@ -7,7 +7,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2018 Insecure.Com LLC ("The Nmap  *
+ * The Nmap Security Scanner is (C) 1996-2019 Insecure.Com LLC ("The Nmap  *
  * Project"). Nmap is also a registered trademark of the Nmap Project.     *
  * This program is free software; you may redistribute and/or modify it    *
  * under the terms of the GNU General Public License as published by the   *
@@ -130,7 +130,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: TargetGroup.cc 37126 2018-01-28 21:18:17Z fyodor $ */
+/* $Id: TargetGroup.cc 37665 2019-06-21 17:00:09Z dmiller $ */
 
 #include "tcpip.h"
 #include "TargetGroup.h"
@@ -861,7 +861,12 @@ int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
      limit it to exactly one address). */
   NetBlock *netblock_resolved = this->netblock->resolve();
   if (netblock_resolved != NULL) {
-    this->netblock = netblock_resolved;
+    /* resolve may return the original netblock if it's not a type that needs
+     * to be resolved. Don't delete it! */
+    if (netblock_resolved != this->netblock) {
+      delete this->netblock;
+      this->netblock = netblock_resolved;
+    }
   }
   else {
     error("Failed to resolve \"%s\".", this->netblock->hostname.c_str());
