@@ -4,28 +4,32 @@
 #include <vector>
 #include <list>
 #include <string>
-#include <string.h>
-#include <iostream>
 
-extern "C" {
-  #include "lua.h"
-  #include "lauxlib.h"
-  #include "lualib.h"
-}
+#include "nse_lua.h"
 
-#include "nmap.h"
-#include "global_structures.h"
+#include "scan_lists.h"
 
 class ScriptResult
 {
   private:
-    std::string output;
     std::string id;
+    /* Structured output table, an integer ref in L_NSE[LUA_REGISTRYINDEX]. */
+    int output_ref;
+    /* Unstructured output string, for scripts that do not return a structured
+       table, or return a string in addition to a table. */
+    std::string output_str;
   public:
-    void set_output (const char *);
-    const char *get_output (void) const;
+    ScriptResult() {
+      output_ref = LUA_NOREF;
+    }
+    void clear (void);
+    void set_output_tab (lua_State *, int);
+    void set_output_str (const char *);
+    void set_output_str (const char *, size_t);
+    std::string get_output_str (void) const;
     void set_id (const char *);
     const char *get_id (void) const;
+    void write_xml() const;
 };
 
 typedef std::list<ScriptResult> ScriptResults;
@@ -38,7 +42,7 @@ class Target;
 
 
 /* API */
-int nse_yield (lua_State *, int, lua_CFunction);
+int nse_yield (lua_State *, lua_KContext, lua_KFunction);
 void nse_restore (lua_State *, int);
 void nse_destructor (lua_State *, char);
 void nse_base (lua_State *);
@@ -63,3 +67,4 @@ void close_nse (void);
 #define SCRIPT_ENGINE_EXTENSION ".nse"
 
 #endif
+
