@@ -2,7 +2,7 @@
 -- Standard Nmap Scripting Engine functions. This module contains various handy
 -- functions that are too small to justify modules of their own.
 --
--- @copyright Same as Nmap--See http://nmap.org/book/man-legal.html
+-- @copyright Same as Nmap--See https://nmap.org/book/man-legal.html
 -- @class module
 -- @name stdnse
 
@@ -10,7 +10,6 @@ local _G = require "_G"
 local coroutine = require "coroutine"
 local math = require "math"
 local nmap = require "nmap"
-local os = require "os"
 local string = require "string"
 local table = require "table"
 local assert = assert;
@@ -25,22 +24,29 @@ local select = select
 local setmetatable = setmetatable;
 local tonumber = tonumber;
 local tostring = tostring;
+local print = print;
 local type = type
+local pcall = pcall
 
 local ceil = math.ceil
 local max = math.max
 
 local format = string.format;
 local rep = string.rep
+local match = string.match
+local find = string.find
+local sub = string.sub
+local gsub = string.gsub
+local char = string.char
+local byte = string.byte
+local gmatch = string.gmatch
 
 local concat = table.concat;
 local insert = table.insert;
+local remove = table.remove;
 local pack = table.pack;
 local unpack = table.unpack;
 
-local difftime = os.difftime;
-local time = os.time;
-
 local EMPTY = {}; -- Empty constant table
 
 _ENV = require "strict" {};
@@ -56,18 +62,60 @@ _ENV = require "strict" {};
 -- @usage stdnse.sleep(1.5)
 _ENV.sleep = nmap.socket.sleep;
 
+-- These stub functions get overwritten by the script run loop in nse_main.lua
+-- These empty stubs will be used if a library calls stdnse.debug while loading
+_ENV.getid = function () return end
+_ENV.getinfo = function () return end
+_ENV.gethostport = function () return end
+
+local function debug (level, ...)
+  if type(level) ~= "number" then
+    return debug(1, level, ...)
+  end
+  local current = nmap.debugging()
+  if level <= current then
+    local host, port = gethostport()
+    local prefix = ( (current >= 2 and getinfo or getid)() or "")
+    .. (host and " "..host.ip .. (port and ":"..port.number or "") or "")
+    if prefix ~= "" then
+      nmap.log_write("stdout", "[" .. prefix .. "] " .. format(...))
+    else
+      nmap.log_write("stdout", format(...))
+    end
+  end
+end
+
 ---
 -- Prints a formatted debug message if the current debugging level is greater
 -- than or equal to a given level.
--- 
--- This is a convenience wrapper around
--- <code>nmap.log_write</code>. The first optional numeric
--- argument, <code>level</code>, is used as the debugging level necessary
--- to print the message (it defaults to 1 if omitted). All remaining arguments
--- are processed with Lua's <code>string.format</code> function.
+--
+-- This is a convenience wrapper around <code>nmap.log_write</code>. The first
+-- optional numeric argument, <code>level</code>, is used as the debugging level
+-- necessary to print the message (it defaults to 1 if omitted). All remaining
+-- arguments are processed with Lua's <code>string.format</code> function.
+--
+-- If known, the output includes some context based information: the script
+-- identifier and the target ip/port (if there is one). If the debug level is
+-- at least 2, it also prints the base thread identifier and whether it is a
+-- worker thread or the master thread.
+--
+-- @class function
+-- @name debug
 -- @param level Optional debugging level.
 -- @param fmt Format string.
 -- @param ... Arguments to format.
+_ENV.debug = debug
+
+--Aliases for particular debug levels
+function debug1 (...) return debug(1, ...) end
+function debug2 (...) return debug(2, ...) end
+function debug3 (...) return debug(3, ...) end
+function debug4 (...) return debug(4, ...) end
+function debug5 (...) return debug(5, ...) end
+
+---
+-- Deprecated version of debug(), kept for now to prevent the script id from being
+-- printed twice. Scripts should use debug() and not pass SCRIPT_NAME
 print_debug = function(level, fmt, ...)
   local l, d = tonumber(level), nmap.debugging();
   if l and l <= d then
@@ -77,18 +125,58 @@ print_debug = function(level, fmt, ...)
   end
 end
 
+local function verbose (level, ...)
+  if type(level) ~= "number" then
+    return verbose(1, level, ...)
+  end
+  local current = nmap.verbosity()
+  if level <= current then
+    local prefix
+    if current >= 2 then
+      local host, port = gethostport()
+      prefix = (getid() or "")
+      .. (host and " "..host.ip .. (port and ":"..port.number or "") or "")
+    else
+      prefix = getid() or ""
+    end
+    if prefix ~= "" then
+      nmap.log_write("stdout", "[" .. prefix .. "] " .. format(...))
+    else
+      nmap.log_write("stdout", format(...))
+    end
+  end
+end
+
 ---
 -- Prints a formatted verbosity message if the current verbosity level is greater
 -- than or equal to a given level.
--- 
--- This is a convenience wrapper around
--- <code>nmap.log_write</code>. The first optional numeric
--- argument, <code>level</code>, is used as the verbosity level necessary
--- to print the message (it defaults to 1 if omitted). All remaining arguments
--- are processed with Lua's <code>string.format</code> function.
+--
+-- This is a convenience wrapper around <code>nmap.log_write</code>. The first
+-- optional numeric argument, <code>level</code>, is used as the verbosity level
+-- necessary to print the message (it defaults to 1 if omitted). All remaining
+-- arguments are processed with Lua's <code>string.format</code> function.
+--
+-- If known, the output includes some context based information: the script
+-- identifier. If the verbosity level is at least 2, it also prints the target
+-- ip/port (if there is one)
+--
+-- @class function
+-- @name verbose
 -- @param level Optional verbosity level.
 -- @param fmt Format string.
 -- @param ... Arguments to format.
+_ENV.verbose = verbose
+
+--Aliases for particular verbosity levels
+function verbose1 (...) return verbose(1, ...) end
+function verbose2 (...) return verbose(2, ...) end
+function verbose3 (...) return verbose(3, ...) end
+function verbose4 (...) return verbose(4, ...) end
+function verbose5 (...) return verbose(5, ...) end
+
+---
+-- Deprecated version of verbose(), kept for now to prevent the script id from being
+-- printed twice. Scripts should use verbose() and not pass SCRIPT_NAME
 print_verbose = function(level, fmt, ...)
   local l, d = tonumber(level), nmap.verbosity();
   if l and l <= d then
@@ -98,75 +186,9 @@ print_verbose = function(level, fmt, ...)
   end
 end
 
-
---- Join a list of strings with a separator string.
--- 
--- This is Lua's <code>table.concat</code> function with the parameters
--- swapped for coherence.
--- @usage
--- stdnse.strjoin(", ", {"Anna", "Bob", "Charlie", "Dolores"})
--- --> "Anna, Bob, Charlie, Dolores"
--- @param delimiter String to delimit each element of the list.
--- @param list Array of strings to concatenate.
--- @return Concatenated string.
-function strjoin(delimiter, list)
-  assert(type(delimiter) == "string" or type(delimiter) == nil, "delimiter is of the wrong type! (did you get the parameters backward?)")
-    
-  return concat(list, delimiter);
-end
-
---- Split a string at a given delimiter, which may be a pattern.
--- @usage
--- stdnse.strsplit(",%s*", "Anna, Bob, Charlie, Dolores")
--- --> { "Anna", "Bob", "Charlie", "Dolores" }
--- @param pattern Pattern that separates the desired strings.
--- @param text String to split.
--- @return Array of substrings without the separating pattern.
-function strsplit(pattern, text)
-  local list, pos = {}, 1;
-
-  assert(pattern ~= "", "delimiter matches empty string!");
-
-  while true do
-    local first, last, match = text:find(pattern, pos);
-    if first then -- found?
-      list[#list+1] = text:sub(pos, first-1);
-      pos = last+1;
-    else
-      list[#list+1] = text:sub(pos);
-      break;
-    end
-  end
-  return list;
-end
-
---- Generate a random string.
--- You can either provide your own charset or the function will use
--- a default one which is [A-Z].
--- @param len Length of the string we want to generate.
--- @param charset Charset that will be used to generate the string.
--- @return A random string of length <code>len</code> consisting of
--- characters from <code>charset</code> if one was provided, otherwise
--- <code>charset</code> defaults to [A-Z] letters.
-function generate_random_string(len, charset)
-  local t = {}
-  local ascii_A = 65
-  local ascii_Z = 90
-  if charset then
-    for i=1,len do
-      t[i]=charset[math.random(#charset)]
-    end
-  else
-    for i=1,len do
-      t[i]=string.char(math.random(ascii_A,ascii_Z))
-    end
-  end
-  return table.concat(t)
-end
-
 --- Return a wrapper closure around a socket that buffers socket reads into
 -- chunks separated by a pattern.
--- 
+--
 -- This function operates on a socket attempting to read data. It separates the
 -- data by <code>sep</code> and, for each invocation, returns a piece of the
 -- separated data. Typically this is used to iterate over the lines of data
@@ -271,9 +293,9 @@ end
 -- stdnse.tohex(123456, {separator = ":"}) --> "1:e2:40"
 -- stdnse.tohex(123456, {separator = ":", group = 4}) --> "1:e240"
 -- @param s String or number to be encoded.
--- @param options Table specifiying formatting options.
+-- @param options Table specifying formatting options.
 -- @return String in hexadecimal format.
-function tohex( s, options ) 
+function tohex( s, options )
   options = options or EMPTY
   local separator = options.separator
   local hex
@@ -302,6 +324,37 @@ function tohex( s, options )
   return hex
 end
 
+---Decode a hexadecimal string to raw bytes
+--
+-- The string can contain any amount of whitespace and capital or lowercase
+-- hexadecimal digits. There must be an even number of hex digits, since it
+-- takes 2 hex digits to make a byte.
+--
+-- @param hex A string in hexadecimal representation
+-- @return A string of bytes or nil if string could not be decoded
+-- @return Error message if string could not be decoded
+function fromhex (hex)
+  local len = #hex
+  local out = {}
+  local i = 1
+  while i <= len do
+    local p, q, c1, c2 = find(hex, "^%s*(%x)%s*(%x)%s*", i)
+    if not p then
+      return nil, format("Invalid characters or odd number of hex digits at %d", i)
+    end
+    out[#out+1] = char(tonumber(c1..c2, 16))
+    i = q + 1
+  end
+  return concat(out)
+end
+
+---Format a MAC address as colon-separated hex bytes.
+--@param mac The MAC address in binary, such as <code>host.mac_addr</code>
+--@return The MAC address in XX:XX:XX:XX:XX:XX format
+function format_mac(mac)
+  return tohex(mac, {separator=":"})
+end
+
 ---Either return the string itself, or return "<blank>" (or the value of the second parameter) if the string
 -- was blank or nil.
 --
@@ -322,8 +375,10 @@ end
 
 ---
 -- Parses a time duration specification, which is a number followed by a
--- unit, and returns a number of seconds. The unit is optional and
--- defaults to seconds. The possible units (case-insensitive) are
+-- unit, and returns a number of seconds.
+--
+-- The unit is optional and defaults to seconds. The possible units
+-- (case-insensitive) are
 -- * <code>ms</code>: milliseconds,
 -- * <code>s</code>: seconds,
 -- * <code>m</code>: minutes,
@@ -347,178 +402,24 @@ function parse_timespec(timespec)
   local n, unit, t, m
   local multipliers = {[""] = 1, s = 1, m = 60, h = 60 * 60, ms = 0.001}
 
-  n, unit = string.match(timespec, "^([%d.]+)(.*)$")
+  n, unit = match(timespec, "^([%d.]+)(.*)$")
   if not n then
-    return nil, string.format("Can't parse time specification \"%s\"", timespec)
+    return nil, format("Can't parse time specification \"%s\"", timespec)
   end
 
   t = tonumber(n)
   if not t then
-    return nil, string.format("Can't parse time specification \"%s\" (bad number \"%s\")", timespec, n)
+    return nil, format("Can't parse time specification \"%s\" (bad number \"%s\")", timespec, n)
   end
 
   m = multipliers[unit]
   if not m then
-    return nil, string.format("Can't parse time specification \"%s\" (bad unit \"%s\")", timespec, unit)
+    return nil, format("Can't parse time specification \"%s\" (bad unit \"%s\")", timespec, unit)
   end
 
   return t * m
 end
 
--- Find the offset in seconds between local time and UTC. That is, if we
--- interpret a UTC date table as a local date table by passing it to os.time,
--- how much must be added to the resulting integer timestamp to make it
--- correct?
-local function utc_offset(t)
-  -- What does the calendar say locally?
-  local localtime = os.date("*t", t)
-  -- What does the calendar say in UTC?
-  local gmtime = os.date("!*t", t)
-  -- Interpret both as local calendar dates and find the difference.
-  return difftime(os.time(localtime), os.time(gmtime))
-end
---- Convert a date table into an integer timestamp. Unlike os.time, this does
--- not assume that the date table represents a local time. Rather, it takes an
--- optional offset number of seconds representing the time zone, and returns
--- the timestamp that would result using that time zone as local time. If the
--- offset is omitted or 0, the date table is interpreted as a UTC date. For
--- example, 4:00 UTC is the same as 5:00 UTC+1:
--- <code>
--- date_to_timestamp({year=1970,month=1,day=1,hour=4,min=0,sec=0})          --> 14400
--- date_to_timestamp({year=1970,month=1,day=1,hour=4,min=0,sec=0}, 0)       --> 14400
--- date_to_timestamp({year=1970,month=1,day=1,hour=5,min=0,sec=0}, 1*60*60) --> 14400
--- </code>
--- And 4:00 UTC+1 is an earlier time:
--- <code>
--- date_to_timestamp({year=1970,month=1,day=1,hour=4,min=0,sec=0}, 1*60*60) --> 10800
--- </code>
-function date_to_timestamp(date, offset)
-  offset = offset or 0
-  return os.time(date) + utc_offset(os.time(date)) - offset
-end
-
-local function format_tz(offset)
-  local sign, hh, mm
-
-  if not offset then
-    return ""
-  end
-  if offset < 0 then
-    sign = "-"
-    offset = -offset
-  else
-    sign = "+"
-  end
-  -- Truncate to minutes.
-  offset = math.floor(offset / 60)
-  hh = math.floor(offset / 60)
-  mm = math.floor(math.fmod(offset, 60))
-
-  return string.format("%s%02d:%02d", sign, hh, mm)
-end
---- Format a date and time (and optional time zone) for structured output.
---
--- Formatting is done according to RFC 3339 (a profile of ISO 8601), except
--- that a time zone may be omitted to signify an unspecified local time zone.
--- Time zones are given as an integer number of seconds from UTC. Use
--- <code>0</code> to mark UTC itself. Formatted strings with a time zone look
--- like this:
--- <code>
--- format_timestamp(os.time(), 0)       --> "2012-09-07T23:37:42+00:00"
--- format_timestamp(os.time(), 2*60*60) --> "2012-09-07T23:37:42+02:00"
--- </code>
--- Without a time zone they look like this:
--- <code>
--- format_timestamp(os.time())          --> "2012-09-07T23:37:42"
--- </code>
---
--- This function should be used for all dates emitted as part of NSE structured
--- output.
-function format_timestamp(t, offset)
-  local tz_string = format_tz(offset)
-  offset = offset or 0
-  return os.date("!%Y-%m-%dT%H:%M:%S", t + offset) .. tz_string
-end
-
---- Format the difference between times <code>t2</code> and <code>t1</code>
--- into a string in one of the forms (signs may vary):
--- * 0s
--- * -4s
--- * +2m38s
--- * -9h12m34s
--- * +5d17h05m06s
--- * -2y177d10h13m20s
--- The string shows <code>t2</code> relative to <code>t1</code>; i.e., the
--- calculation is <code>t2</code> minus <code>t1</code>.
-function format_difftime(t2, t1)
-  local d, s, sign, yeardiff
-
-  d = difftime(time(t2), time(t1))
-  if d > 0 then
-    sign = "+"
-  elseif d < 0 then
-    sign = "-"
-    t2, t1 = t1, t2
-    d = -d
-  else
-    sign = ""
-  end
-  -- t2 is always later than or equal to t1 here.
-
-  -- The year is a tricky case because it's not a fixed number of days
-  -- the way a day is a fixed number of hours or an hour is a fixed
-  -- number of minutes. For example, the difference between 2008-02-10
-  -- and 2009-02-10 is 366 days because 2008 was a leap year, but it
-  -- should be printed as 1y0d0h0m0s, not 1y1d0h0m0s. We advance t1 to be
-  -- the latest year such that it is still before t2, which means that its
-  -- year will be equal to or one less than t2's. The number of years
-  -- skipped is stored in yeardiff.
-  if t2.year > t1.year then
-    local tmpyear = t1.year
-    -- Put t1 in the same year as t2.
-    t1.year = t2.year
-    d = difftime(time(t2), time(t1))
-    if d < 0 then
-      -- Too far. Back off one year.
-      t1.year = t2.year - 1
-      d = difftime(time(t2), time(t1))
-    end
-    yeardiff = t1.year - tmpyear
-    t1.year = tmpyear
-  else
-    yeardiff = 0
-  end
-
-  local s, sec, min
-  s = ""
-  -- Seconds (pad to two digits).
-  sec = d % 60
-  d = math.floor(d / 60)
-  if d == 0 and yeardiff == 0 then
-    return sign .. string.format("%gs", sec) .. s
-  end
-  s = string.format("%02gs", sec) .. s
-  -- Minutes (pad to two digits).
-  min = d % 60
-  d = math.floor(d / 60)
-  if d == 0 and yeardiff == 0 then
-    return sign .. string.format("%dm", min) .. s
-  end
-  s = string.format("%02dm", min) .. s
-  -- Hours.
-  s = string.format("%dh", d % 24) .. s
-  d = math.floor(d / 24)
-  if d == 0 and yeardiff == 0 then
-    return sign .. s
-  end
-  -- Days.
-  s = string.format("%dd", d) .. s
-  if yeardiff == 0 then return sign .. s end
-  -- Years.
-  s = string.format("%dy", yeardiff) .. s
-  return sign .. s
-end
-
 --- Returns the current time in milliseconds since the epoch
 -- @return The current time in milliseconds since the epoch
 function clock_ms()
@@ -531,49 +432,11 @@ function clock_us()
   return nmap.clock() * 1000000
 end
 
----Get the indentation symbols at a given level. 
-local function format_get_indent(indent, at_end)
-  local str = ""
-  local had_continue = false
-
-  if(not(at_end)) then
-    str = rep('  ', #indent) -- Was: "|  "
-  else
-    for i = #indent, 1, -1 do
-      if(indent[i] and not(had_continue)) then
-        str = str .. "  " -- Was: "|_ "
-      else
-        had_continue = true
-        str = str .. "  " -- Was: "|  "
-      end
-    end
-  end
-
-  return str
+---Get the indentation symbols at a given level.
+local function format_get_indent(indent)
+  return rep("  ", #indent)
 end
 
-local function splitlines(s)
-  local result = {}
-  local i = 0
-
-  while i <= #s do
-    local b, e
-    b, e = string.find(s, "\r?\n", i)
-    if not b then
-      break
-    end
-    result[#result + 1] = string.sub(s, i, b - 1)
-    i = e + 1
-  end
-
-  if i <= #s then
-    result[#result + 1] = string.sub(s, i)
-  end
-
-  return result
-end
-
-
 -- A helper for format_output (see below).
 local function format_output_sub(status, data, indent)
   if (#data == 0) then
@@ -634,14 +497,14 @@ local function format_output_sub(status, data, indent)
       end
 
       insert(output, format_output_sub(status, value, new_indent))
-        
-    elseif(type(value) == 'string') then
-      local lines = splitlines(value)
 
-      for j, line in ipairs(lines) do
-      	insert(output, format("%s  %s%s\n",
-      	                  format_get_indent(indent, i == #data and j == #lines),
-      	                  prefix, line))
+    elseif(type(value) == 'string') then
+      -- ensure it ends with a newline
+      if sub(value, -1) ~= "\n" then value = value .. "\n" end
+      for line in gmatch(value, "([^\r\n]-)\n") do
+        insert(output, format("%s  %s%s\n",
+          format_get_indent(indent),
+          prefix, line))
       end
     end
   end
@@ -649,30 +512,36 @@ local function format_output_sub(status, data, indent)
   return concat(output)
 end
 
----Takes a table of output on the commandline and formats it for display to the 
--- user. This is basically done by converting an array of nested tables into a 
--- string. In addition to numbered array elements, each table can have a 'name' 
--- and a 'warning' value. The 'name' will be displayed above the table, and 
+---This function is deprecated.
+--
+-- Please use structured NSE output instead: https://nmap.org/book/nse-api.html#nse-structured-output
+--
+-- Takes a table of output on the commandline and formats it for display to the
+-- user.
+--
+-- This is basically done by converting an array of nested tables into a
+-- string. In addition to numbered array elements, each table can have a 'name'
+-- and a 'warning' value. The 'name' will be displayed above the table, and
 -- 'warning' will be displayed, with a 'WARNING' tag, if and only if debugging
--- is enabled. 
--- 
+-- is enabled.
+--
 -- Here's an example of a table:
 -- <code>
 --   local domains = {}
 --   domains['name'] = "DOMAINS"
 --   table.insert(domains, 'Domain 1')
 --   table.insert(domains, 'Domain 2')
--- 
+--
 --   local names = {}
 --   names['name'] = "NAMES"
 --   names['warning'] = "Not all names could be determined!"
 --   table.insert(names, "Name 1")
--- 
+--
 --   local response = {}
 --   table.insert(response, "Apple pie")
 --   table.insert(response, domains)
 --   table.insert(response, names)
--- 
+--
 --   return stdnse.format_output(true, response)
 -- </code>
 --
@@ -688,13 +557,13 @@ end
 --   |_     Name 1
 -- </code>
 --
---@param status A boolean value dictating whether or not the script succeeded. 
+--@param status A boolean value dictating whether or not the script succeeded.
 --              If status is false, and debugging is enabled, 'ERROR' is prepended
 --              to every line. If status is false and debugging is disabled, no output
---              occurs. 
---@param data   The table of output. 
+--              occurs.
+--@param data   The table of output.
 --@param indent Used for indentation on recursive calls; should generally be set to
---              nil when callling from a script. 
+--              nil when calling from a script.
 -- @return <code>nil</code>, if <code>data</code> is empty, otherwise a
 -- multiline string.
 function format_output(status, data, indent)
@@ -720,23 +589,33 @@ end
 -- given. This works also for arguments given as top-level array values, like
 -- --script-args=unsafe; for these it returns the value 1.
 local function arg_value(argname)
+  -- First look for the literal script-arg name
+  -- as a key/value pair
   if nmap.registry.args[argname] then
     return nmap.registry.args[argname]
-  else
-    -- if scriptname.arg is not there, check "arg"
-    local argument_frags = strsplit("%.", argname)
-    if #argument_frags > 0 then
-      if nmap.registry.args[argument_frags[2]] then
-        return nmap.registry.args[argument_frags[2]]
-      end
-    end
   end
-
+  -- and alone, as a boolean flag
   for _, v in ipairs(nmap.registry.args) do
     if v == argname then
       return 1
     end
   end
+
+  -- if scriptname.arg is not there, check "arg"
+  local shortname = argname:match("%.([^.]*)$")
+  if shortname then
+    -- as a key/value pair
+    if nmap.registry.args[shortname] then
+      return nmap.registry.args[shortname]
+    end
+    -- and alone, as a boolean flag
+    for _, v in ipairs(nmap.registry.args) do
+      if v == shortname then
+        return 1
+      end
+    end
+  end
+  return nil
 end
 
 --- Parses the script arguments passed to the --script-args option.
@@ -764,7 +643,7 @@ end
 function get_script_args (...)
   local args = {}
 
-  for i, set in ipairs({...}) do 
+  for i, set in ipairs({...}) do
     if type(set) == "string" then
       set = {set}
     end
@@ -780,10 +659,10 @@ function get_script_args (...)
   return unpack(args, 1, select("#", ...))
 end
 
----Get the best possible hostname for the given host. This can be the target as given on 
--- the commandline, the reverse dns name, or simply the ip address. 
---@param host The host table (or a string that'll simply be returned). 
---@return The best possible hostname, as a string. 
+---Get the best possible hostname for the given host. This can be the target as given on
+-- the commandline, the reverse dns name, or simply the ip address.
+--@param host The host table (or a string that'll simply be returned).
+--@return The best possible hostname, as a string.
 function get_hostname(host)
   if type(host) == "table" then
     return host.targetname or ( host.name ~= '' and host.name ) or host.ip
@@ -793,7 +672,7 @@ function get_hostname(host)
 end
 
 ---Retrieve an item from the registry, checking if each sub-key exists. If any key doesn't
--- exist, return nil. 
+-- exist, return nil.
 function registry_get(subkeys)
   local registry = nmap.registry
   local i = 1
@@ -811,7 +690,7 @@ function registry_get(subkeys)
   return registry
 end
 
---Check if the given element exists in the registry. If 'key' is nil, it isn't checked. 
+--Check if the given element exists in the registry. If 'key' is nil, it isn't checked.
 function registry_exists(subkeys, key, value)
   local subkey = registry_get(subkeys)
 
@@ -828,12 +707,13 @@ function registry_exists(subkeys, key, value)
   return false
 end
 
----Add an item to an array in the registry, creating all sub-keys if necessary. 
+---Add an item to an array in the registry, creating all sub-keys if necessary.
+--
 -- For example, calling:
 -- <code>registry_add_array({'192.168.1.100', 'www', '80', 'pages'}, 'index.html')</code>
 -- Will create nmap.registry['192.168.1.100'] as a table, if necessary, then add a table
 -- under the 'www' key, and so on. 'pages', finally, is treated as an array and the value
--- given is added to the end. 
+-- given is added to the end.
 function registry_add_array(subkeys, value, allow_duplicates)
   local registry = nmap.registry
   local i = 1
@@ -863,7 +743,7 @@ function registry_add_array(subkeys, value, allow_duplicates)
 end
 
 ---Similar to <code>registry_add_array</code>, except instead of adding a value to the
--- end of an array, it adds a key:value pair to the table. 
+-- end of an array, it adds a key:value pair to the table.
 function registry_add_table(subkeys, key, value, allow_duplicates)
   local registry = nmap.registry
   local i = 1
@@ -1002,76 +882,6 @@ do end -- no function here, see nse_main.lua
 do end -- no function here, see nse_main.lua
 
 
-
----Checks if the port is in the port range
--- For example, calling:
--- <code>in_port_range({number=31337,protocol="udp"},"T:15,50-75,U:31334-31339")</code>
--- would result in a true value
---@param port a port structure containing keys port number(number) and protocol(string)
---@param port_range a port range string in Nmap standard format (ex. "T:80,1-30,U:31337,21-25")
---@returns boolean indicating whether the port is in the port range
-function in_port_range(port,port_range)
-	assert(port and type(port.number)=="number" and type(port.protocol)=="string" and 
-			(port.protocol=="udp" or port.protocol=="tcp"),"Port structure missing or invalid: port={ number=<port_number>, protocol=<port_protocol> }")
-	assert((type(port_range)=="string" or type(port_range)=="number") and port_range~="","Incorrect port range specification.")
-	
-	-- Proto - true for TCP, false for UDP
-	local proto
-	if(port.protocol=="tcp") then proto = true else proto = false end
-	
-	--TCP flag for iteration - true for TCP, false for UDP, if not specified we presume TCP
-	local tcp_flag = true
-	
-	-- in case the port_range is a single number 
-	if type(port_range)=="number" then
-		if proto and port_range==port.number then return true
-		else return false
-		end
-	end
-	
-	--clean the string a bit
-	port_range=port_range:gsub("%s+","")
-	
-	-- single_pr - single port range
-	for i, single_pr in ipairs(strsplit(",",port_range)) do
-		if single_pr:match("T:") then
-			tcp_flag = true
-			single_pr = single_pr:gsub("T:","")
-		else 
-			if single_pr:match("U:") then 
-				tcp_flag = false 
-				single_pr = single_pr:gsub("U:","")
-			end
-		end
-		
-		-- compare ports only when the port's protocol is the same as
-		-- the current single port range
-		if tcp_flag == proto then
-			local pone = single_pr:match("^(%d+)$")
-			if pone then
-				pone = tonumber(pone)
-				assert(pone>-1 and pone<65536, "Port range number out of range (0-65535).")
-				
-				if pone == port.number then 
-					return true 
-				end
-			else
-				local pstart, pend = single_pr:match("^(%d+)%-(%d+)$")
-				pstart, pend = tonumber(pstart), tonumber(pend)
-				assert(pstart,"Incorrect port range specification.")
-				assert(pstart<=pend,"Incorrect port range specification, the starting port should have a smaller value than the ending port.")
-				assert(pstart>-1 and pstart<65536 and pend>-1 and pend<65536, "Port range number out of range (0-65535).")
-				
-				if port.number >=pstart and port.number <= pend then
-					return true
-				end
-			end
-		end
-	end
-	-- if no match is found then the port doesn't belong to the port_range
-	return false
-end
-
 --- Module function that mimics some behavior of Lua 5.1 module function.
 --
 -- This convenience function returns a module environment to set the _ENV
@@ -1135,12 +945,12 @@ function output_table ()
     __newindex = function (_, k, v)
       if t[k] == nil and v ~= nil then
         -- New key?
-        table.insert(order, k)
+        insert(order, k)
       elseif v == nil then
         -- Deleting an existing key?
         for i, key in ipairs(order) do
           if key == k then
-            table.remove(order, i)
+            remove(order, i)
             break
           end
         end
@@ -1155,6 +965,9 @@ function output_table ()
     end,
     __call = function (_) -- hack to mean "not_empty?"
       return not not next(order)
+    end,
+    __len = function (_)
+      return #order
     end
   }
   return setmetatable({}, mt)
@@ -1166,8 +979,8 @@ end
 -- printer function. The printer function takes a sole string
 -- argument and will be called repeatedly.
 --
--- @args obj The object to pretty print.
--- @args printer The printer function.
+-- @param obj The object to pretty print.
+-- @param printer The printer function.
 function pretty_printer (obj, printer)
   if printer == nil then printer = print end
 
@@ -1195,36 +1008,40 @@ function pretty_printer (obj, printer)
   return aux(obj, "")
 end
 
--- This pattern must match the percent sign '%' since it is used in
--- escaping.
-local FILESYSTEM_UNSAFE = "[^a-zA-Z0-9._-]"
----
--- Escape a string to remove bytes and strings that may have meaning to
--- a filesystem, such as slashes. All bytes are escaped, except for:
--- * alphabetic <code>a</code>-<code>z</code> and <code>A</code>-<code>Z</code>, digits 0-9, <code>.</code> <code>_</code> <code>-</code>
--- In addition, the strings <code>"."</code> and <code>".."</code> have
--- their characters escaped.
+--- Returns a conservative timeout for a host
 --
--- Bytes are escaped by a percent sign followed by the two-digit
--- hexadecimal representation of the byte value.
--- * <code>filename_escape("filename.ext") --> "filename.ext"</code>
--- * <code>filename_escape("input/output") --> "input%2foutput"</code>
--- * <code>filename_escape(".") --> "%2e"</code>
--- * <code>filename_escape("..") --> "%2e%2e"</code>
--- This escaping is somewhat like that of JavaScript
--- <code>encodeURIComponent</code>, except that fewer bytes are
--- whitelisted, and it works on bytes, not Unicode characters or UTF-16
--- code points.
-function filename_escape(s)
-  if s == "." then
-    return "%2e"
-  elseif s == ".." then
-    return "%2e%2e"
-  else
-    return (string.gsub(s, FILESYSTEM_UNSAFE, function (c)
-      return string.format("%%%02x", string.byte(c))
-    end))
+-- If the host parameter is a NSE host table with a <code>times.timeout</code>
+-- attribute, then the return value is the host timeout scaled according to the
+-- max_timeout. The scaling factor is defined by a linear formula such that
+-- (max_timeout=8000, scale=2) and (max_timeout=1000, scale=1)
+--
+-- @param host The host object to base the timeout on. If this is anything but
+--             a host table, the max_timeout is returned.
+-- @param max_timeout The maximum timeout in milliseconds. This is the default
+--                    timeout used if there is no host.times.timeout. Default: 8000
+-- @param min_timeout The minimum timeout in milliseconds that will be
+--                    returned. Default: 1000
+-- @return The timeout in milliseconds, suitable for passing to set_timeout
+-- @usage
+-- assert(host.times.timeout == 1.3)
+--  assert(get_timeout() == 8000)
+--  assert(get_timeout(nil, 5000) == 5000)
+--  assert(get_timeout(host) == 2600)
+--  assert(get_timeout(host, 10000, 3000) == 3000)
+function get_timeout(host, max_timeout, min_timeout)
+  max_timeout = max_timeout or 8000
+  local t = type(host) == "table" and host.times and host.times.timeout
+  if not t then
+    return max_timeout
   end
+  t = t * (max_timeout + 6000) / 7
+  min_timeout = min_timeout or 1000
+  if t < min_timeout then
+    return min_timeout
+  elseif t > max_timeout then
+    return max_timeout
+  end
+  return t
 end
 
 return _ENV;
