@@ -1,11 +1,10 @@
-
 /***************************************************************************
  * netutils.c -- This contains some useful little network/socket related   *
  * utility functions.                                                      *
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2011 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2019 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -29,22 +28,22 @@
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -54,7 +53,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: netutils.c 21905 2011-01-21 00:04:51Z fyodor $ */
+/* $Id: netutils.c 37640 2019-05-28 21:36:04Z dmiller $ */
 
 #include "netutils.h"
 #include "error.h"
@@ -84,20 +83,21 @@
 #if HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
+#if HAVE_SYS_UN_H
+#include <sys/un.h>
+#endif
 
 static int netutils_debugging = 0;
 
-/* maximize the number of file descriptors (including sockets) allowed
-   for this process and return that maximum value (note -- you better
-   not actually open this many -- stdin, stdout, other files opened by
-   libraries you use, etc. all count toward this limit.  Leave a
-   little slack */
+/* maximize the number of file descriptors (including sockets) allowed for this
+ * process and return that maximum value (note -- you better not actually open
+ * this many -- stdin, stdout, other files opened by libraries you use, etc. all
+ * count toward this limit.  Leave a little slack */
+int maximize_fdlimit(void) {
 
-int maximize_fdlimit() { 
 #ifndef WIN32
-
-struct rlimit r; 
-static int maxfds = -1;
+  struct rlimit r;
+  static int maxfds = -1;
 
   if (maxfds > 0)
     return maxfds;
@@ -106,25 +106,92 @@ static int maxfds = -1;
   if (!getrlimit(RLIMIT_NOFILE, &r)) {
     r.rlim_cur = r.rlim_max;
     if (setrlimit(RLIMIT_NOFILE, &r))
-      if (netutils_debugging) perror("setrlimit RLIMIT_NOFILE failed");
+      if (netutils_debugging)
+        perror("setrlimit RLIMIT_NOFILE failed");
+
     if (!getrlimit(RLIMIT_NOFILE, &r)) {
       maxfds = r.rlim_cur;
       return maxfds;
-    } else return 0;
+    } else {
+      return 0;
+    }
   }
 #endif
+
 #if(defined(RLIMIT_OFILE) && !defined(RLIMIT_NOFILE))
   if (!getrlimit(RLIMIT_OFILE, &r)) {
     r.rlim_cur = r.rlim_max;
     if (setrlimit(RLIMIT_OFILE, &r))
-      if (netutils_debugging) perror("setrlimit RLIMIT_OFILE failed");
+      if (netutils_debugging)
+        perror("setrlimit RLIMIT_OFILE failed");
+
     if (!getrlimit(RLIMIT_OFILE, &r)) {
       maxfds = r.rlim_cur;
       return maxfds;
-    } else return 0;
+    } else {
+      return 0;
+    }
   }
 #endif
 #endif /* !WIN32 */
   return 0;
 }
 
+#if HAVE_SYS_UN_H
+  #define PEER_STR_LEN sizeof(((struct sockaddr_un *) 0)->sun_path)
+#else
+  #define PEER_STR_LEN sizeof("[ffff:ffff:ffff:ffff:ffff:ffff:255.255.255.255]:xxxxx")
+#endif
+
+#if HAVE_SYS_UN_H
+/* Get the UNIX domain socket path or empty string if the address family != AF_UNIX. */
+const char *get_unixsock_path(const struct sockaddr_storage *addr) {
+  struct sockaddr_un *su = (struct sockaddr_un *)addr;
+
+  if (!addr || addr->ss_family != AF_UNIX)
+    return "";
+
+  return (const char *)su->sun_path;
+}
+#endif
+
+static int get_port(const struct sockaddr_storage *ss) {
+  if (ss->ss_family == AF_INET)
+    return ntohs(((struct sockaddr_in *) ss)->sin_port);
+#if HAVE_IPV6
+  else if (ss->ss_family == AF_INET6)
+    return ntohs(((struct sockaddr_in6 *) ss)->sin6_port);
+#endif
+
+  return -1;
+}
+
+static char *get_addr_string(const struct sockaddr_storage *ss, size_t sslen) {
+  static char buffer[PEER_STR_LEN];
+
+#if HAVE_SYS_UN_H
+  if (ss->ss_family == AF_UNIX) {
+    sprintf(buffer, "%s", get_unixsock_path(ss));
+    return buffer;
+  }
+#endif
+
+  sprintf(buffer, "%s:%d", inet_ntop_ez(ss, sslen), get_port(ss));
+  return buffer;
+}
+
+/* Get the peer/host address string.
+ * In case we have support for UNIX domain sockets, function returns
+ * string containing path to UNIX socket if the address family is AF_UNIX,
+ * otherwise it returns string containing "<address>:<port>". */
+char *get_peeraddr_string(const struct niod *iod) {
+  if (iod->peerlen > 0)
+    return get_addr_string(&iod->peer, iod->peerlen);
+  else
+    return "peer unspecified";
+}
+
+/* Get the local bind address string. */
+char *get_localaddr_string(const struct niod *iod) {
+  return get_addr_string(&iod->local, iod->locallen);
+}
