@@ -1,13 +1,13 @@
 /***************************************************************************
  * nsock_event.c -- Functions dealing with nsock_events (and their         *
- * msevent internal representation.  An event is created when you do       *
+ * struct nevent internal representation.  An event is created when you do       *
  * various calls (for reading, writing, connecting, timers, etc) and is    *
  * provided back to you in the callback when the call completes or         *
  * fails. It is automatically destroyed after the callback returns         *
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2012 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2019 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -31,22 +31,22 @@
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -56,9 +56,10 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_event.c 28387 2012-04-03 00:21:44Z david $ */
+/* $Id: nsock_event.c 37640 2019-05-28 21:36:04Z dmiller $ */
 
 #include "nsock_internal.h"
+#include "nsock_log.h"
 #include "gh_list.h"
 
 #if HAVE_PCAP
@@ -71,38 +72,38 @@ extern struct timeval nsock_tod;
 
 /* Find the type of an event that spawned a callback */
 enum nse_type nse_type(nsock_event nse) {
-  msevent *me = (msevent *)nse;
+  struct nevent *me = (struct nevent *)nse;
   return me->type;
 }
 
 enum nse_status nse_status(nsock_event nse) {
-  msevent *me = (msevent *)nse;
+  struct nevent *me = (struct nevent *)nse;
   return me->status;
 }
 
 int nse_eof(nsock_event nse) {
-  msevent *me = (msevent *)nse;
+  struct nevent *me = (struct nevent *)nse;
   return me->eof;
 }
 
 /* Obtains the nsock_iod (see below) associated with the event.  Note that
  * some events (such as timers) don't have an nsock_iod associated with them */
 nsock_iod nse_iod(nsock_event ms_event) {
-  msevent *nse = (msevent *)ms_event;
+  struct nevent *nse = (struct nevent *)ms_event;
   return (nsock_iod) nse->iod;
 }
 
 /* This next function returns the errno style error code -- which is only valid
  * if the status is NSE_STATUS_ERROR */
 int nse_errorcode(nsock_event nse) {
-  msevent *me = (msevent *)nse;
+  struct nevent *me = (struct nevent *)nse;
   return me->errnum;
 }
 
 /* Every event has an ID which will be unique throughout the program's execution
  * unless you use (literally) billions of them */
 nsock_event_id nse_id(nsock_event nse) {
-  msevent *me = (msevent *)nse;
+  struct nevent *me = (struct nevent *)nse;
   return me->id;
 }
 
@@ -110,25 +111,30 @@ nsock_event_id nse_id(nsock_event nse) {
  * provides the buffer that was read in as well as the number of chars read.
  * The buffer should not be modified or free'd */
 char *nse_readbuf(nsock_event nse, int *nbytes) {
-  msevent *me = (msevent *)nse;
+  struct nevent *me = (struct nevent *)nse;
 
   if (nbytes)
-    *nbytes = FILESPACE_LENGTH(&(me->iobuf));
-  return FILESPACE_STR(&(me->iobuf));
+    *nbytes = fs_length(&(me->iobuf));
+  return fs_str(&(me->iobuf));
 }
 
-static void first_ev_next(msevent *nse, gh_list_elem **first) {
+static void first_ev_next(struct nevent *nse, gh_lnode_t **first, int nodeq2) {
   if (!first || !*first)
     return;
 
-  if ((msevent *)GH_LIST_ELEM_DATA(*first) == nse) {
-    gh_list_elem *next;
+  if (&nse->nodeq_io == *first || &nse->nodeq_pcap == *first) {
+    gh_lnode_t *next;
 
-    next = GH_LIST_ELEM_NEXT(*first);
+    next = gh_lnode_next(*first);
     if (next) {
-      msevent *nse2 = (msevent *)GH_LIST_ELEM_DATA(next);
+      struct nevent *newevent;
 
-      if (nse2->iod == nse->iod)
+      if (nodeq2)
+        newevent = lnode_nevent2(next);
+      else
+        newevent = lnode_nevent(next);
+
+      if (newevent->iod == nse->iod)
         *first = next;
       else
         *first = NULL;
@@ -138,25 +144,25 @@ static void first_ev_next(msevent *nse, gh_list_elem **first) {
   }
 }
 
-void update_first_events(msevent *nse) {
-  switch(get_event_id_type(nse->id)) {
+void update_first_events(struct nevent *nse) {
+  switch (get_event_id_type(nse->id)) {
     case NSE_TYPE_CONNECT:
     case NSE_TYPE_CONNECT_SSL:
-      first_ev_next(nse, &nse->iod->first_connect);
+      first_ev_next(nse, &nse->iod->first_connect, 0);
       break;
 
     case NSE_TYPE_READ:
-      first_ev_next(nse, &nse->iod->first_read);
+      first_ev_next(nse, &nse->iod->first_read, 0);
       break;
 
     case NSE_TYPE_WRITE:
-      first_ev_next(nse, &nse->iod->first_write);
+      first_ev_next(nse, &nse->iod->first_write, 0);
       break;
 
 #if HAVE_PCAP
     case NSE_TYPE_PCAP_READ:
-      first_ev_next(nse, &nse->iod->first_read);
-      first_ev_next(nse, &nse->iod->first_pcap_read);
+      first_ev_next(nse, &nse->iod->first_read, 0);
+      first_ev_next(nse, &nse->iod->first_pcap_read, 1);
       break;
 #endif
 
@@ -176,23 +182,23 @@ void update_first_events(msevent *nse) {
  * it is the one which created it), in which case 0 can be passed to skip the
  * step.  This function returns zero if the event is not found, nonzero
  * otherwise. */
-int nsock_event_cancel(nsock_pool ms_pool, nsock_event_id id, int notify ) {
-  mspool *nsp = (mspool *)ms_pool;
+int nsock_event_cancel(nsock_pool ms_pool, nsock_event_id id, int notify) {
+  struct npool *nsp = (struct npool *)ms_pool;
   enum nse_type type;
-  gh_list *event_list = NULL, *event_list2 = NULL;
-  gh_list_elem *current, *next;
-  msevent *nse = NULL;
+  unsigned int i;
+  gh_list_t *event_list = NULL, *event_list2 = NULL;
+  gh_lnode_t *current, *next;
+  struct nevent *nse = NULL;
 
   assert(nsp);
 
   type = get_event_id_type(id);
-  if (nsp->tracelevel > 0) {
-    nsock_trace(nsp, "Event #%li (type %s) cancelled", id, nse_type2str(type));
-  }
+  nsock_log_info("Event #%li (type %s) cancelled", id, nse_type2str(type));
 
   /* First we figure out what list it is in */
-  switch(type) {
+  switch (type) {
     case NSE_TYPE_CONNECT:
+    case NSE_TYPE_CONNECT_SSL:
       event_list = &nsp->connect_events;
       break;
 
@@ -205,8 +211,15 @@ int nsock_event_cancel(nsock_pool ms_pool, nsock_event_id id, int notify ) {
       break;
 
     case NSE_TYPE_TIMER:
-      event_list = &nsp->timer_events;
-      break;
+      for (i = 0; i < gh_heap_count(&nsp->expirables); i++) {
+        gh_hnode_t *hnode;
+
+        hnode = gh_heap_find(&nsp->expirables, i);
+        nse = container_of(hnode, struct nevent, expire);
+        if (nse->id == id)
+          return nevent_delete(nsp, nse, NULL, NULL, notify);
+      }
+      return 0;
 
 #if HAVE_PCAP
     case NSE_TYPE_PCAP_READ:
@@ -220,18 +233,18 @@ int nsock_event_cancel(nsock_pool ms_pool, nsock_event_id id, int notify ) {
   }
 
   /* Now we try to find the event in the list */
-  for (current = GH_LIST_FIRST_ELEM(event_list); current != NULL;  current = next) {
-    next = GH_LIST_ELEM_NEXT(current);
-    nse = (msevent *)GH_LIST_ELEM_DATA(current);
+  for (current = gh_list_first_elem(event_list); current != NULL; current = next) {
+    next = gh_lnode_next(current);
+    nse = lnode_nevent(current);
     if (nse->id == id)
       break;
   }
 
-  if (current == NULL && event_list2){
+  if (current == NULL && event_list2) {
     event_list = event_list2;
-    for (current = GH_LIST_FIRST_ELEM(event_list); current != NULL; current = next) {
-      next = GH_LIST_ELEM_NEXT(current);
-      nse = (msevent *)GH_LIST_ELEM_DATA(current);
+    for (current = gh_list_first_elem(event_list); current != NULL; current = next) {
+      next = gh_lnode_next(current);
+      nse = lnode_nevent2(current);
       if (nse->id == id)
         break;
     }
@@ -239,17 +252,18 @@ int nsock_event_cancel(nsock_pool ms_pool, nsock_event_id id, int notify ) {
   if (current == NULL)
     return 0;
 
-  return msevent_cancel(nsp, nse, event_list, current, notify);
+  return nevent_delete(nsp, nse, event_list, current, notify);
 }
 
 /* An internal function for cancelling an event when you already have a pointer
- * to the msevent (use nsock_event_cancel if you just have an ID). The
+ * to the struct nevent (use nsock_event_cancel if you just have an ID). The
  * event_list passed in should correspond to the type of the event. For example,
  * with NSE_TYPE_READ, you would pass in &nsp->read_events;. elem is the list
  * element in event_list which holds the event.  Pass a nonzero for notify if
  * you want the program owning the event to be notified that it has been
  * cancelled */
-int msevent_cancel(mspool *nsp, msevent *nse, gh_list *event_list, gh_list_elem *elem, int notify) {
+int nevent_delete(struct npool *nsp, struct nevent *nse, gh_list_t *event_list,
+                   gh_lnode_t *elem, int notify) {
   if (nse->event_done) {
     /* This event has already been marked for death somewhere else -- it will be
      * gone soon (and if we try to kill it now all hell will break loose due to
@@ -257,12 +271,12 @@ int msevent_cancel(mspool *nsp, msevent *nse, gh_list *event_list, gh_list_elem
     return 0;
   }
 
-  if (nsp->tracelevel > 0)
-    nsock_trace(nsp, "msevent_cancel() on event #%li (type %s)", nse->id, nse_type2str(nse->type));
+  nsock_log_info("%s on event #%li (type %s)", __func__, nse->id,
+                 nse_type2str(nse->type));
 
   /* Now that we found the event... we go through the motions of cleanly
    * cancelling it */
-  switch(nse->type) {
+  switch (nse->type) {
     case NSE_TYPE_CONNECT:
     case NSE_TYPE_CONNECT_SSL:
       handle_connect_result(nsp, nse, NSE_STATUS_CANCELLED);
@@ -287,23 +301,29 @@ int msevent_cancel(mspool *nsp, msevent *nse, gh_list *event_list, gh_list_elem
 #endif
 
     default:
-      assert(0);
+      fatal("Invalid nsock event type (%d)", nse->type);
   }
 
   assert(nse->event_done);
-  update_first_events(nse);
-  gh_list_remove_elem(event_list, elem);
 
-  if (nsp->tracelevel > 8)
-    nsock_trace(nsp, "NSE #%lu: Removing event from list", nse->id);
+  if (nse->timeout.tv_sec)
+    gh_heap_remove(&nsp->expirables, &nse->expire);
+
+  if (event_list) {
+    update_first_events(nse);
+    gh_list_remove(event_list, elem);
+  }
+
+  gh_list_append(&nsp->free_events, &nse->nodeq_io);
+
+  nsock_log_debug_all("NSE #%lu: Removing event from list", nse->id);
 
 #if HAVE_PCAP
 #if PCAP_BSD_SELECT_HACK
   if (nse->type == NSE_TYPE_PCAP_READ) {
-    if (nsp->tracelevel > 8)
-      nsock_trace(nsp, "PCAP NSE #%lu: CANCEL TEST pcap=%p read=%p curr=%p sd=%i",
-                  nse->id, &nsp->pcap_read_events, &nsp->read_events,
-                  event_list,((mspcap *)nse->iod->pcap)->pcap_desc);
+    nsock_log_debug_all("PCAP NSE #%lu: CANCEL TEST pcap=%p read=%p curr=%p sd=%i",
+                        nse->id, &nsp->pcap_read_events, &nsp->read_events,
+                        event_list,((mspcap *)nse->iod->pcap)->pcap_desc);
 
     /* If event occurred, and we're in BSD_HACK mode, then this event was added to
      * two queues. read_event and pcap_read_event Of course we should
@@ -312,23 +332,21 @@ int msevent_cancel(mspool *nsp, msevent *nse, gh_list *event_list, gh_list_elem
     if (((mspcap *)nse->iod->pcap)->pcap_desc >= 0 && event_list == &nsp->read_events) {
       /* event is done, list is read_events and we're in BSD_HACK mode. So unlink
        * event from pcap_read_events */
-      gh_list_remove(&nsp->pcap_read_events, nse);
-      if (nsp->tracelevel > 8)
-        nsock_trace(nsp, "PCAP NSE #%lu: Removing event from PCAP_READ_EVENTS", nse->id);
+      gh_list_remove(&nsp->pcap_read_events, &nse->nodeq_pcap);
+      nsock_log_debug_all("PCAP NSE #%lu: Removing event from PCAP_READ_EVENTS", nse->id);
     }
 
     if (((mspcap *)nse->iod->pcap)->pcap_desc >= 0 && event_list == &nsp->pcap_read_events) {
       /* event is done, list is read_events and we're in BSD_HACK mode.
        * So unlink event from read_events */
-      gh_list_remove(&nsp->read_events, nse);
+      gh_list_remove(&nsp->read_events, &nse->nodeq_io);
 
-      if (nsp->tracelevel > 8)
-        nsock_trace(nsp, "PCAP NSE #%lu: Removing event from READ_EVENTS", nse->id);
+      nsock_log_debug_all("PCAP NSE #%lu: Removing event from READ_EVENTS", nse->id);
     }
   }
 #endif
 #endif
-  msevent_dispatch_and_delete(nsp, nse, notify);
+  event_dispatch_and_delete(nsp, nse, notify);
   return 1;
 }
 
@@ -336,7 +354,7 @@ int msevent_cancel(mspool *nsp, msevent *nse, gh_list *event_list, gh_list_elem
  * nonzero) and then deletes the event.  This function does NOT delete the event
  * from any lists it might be on (eg nsp->read_list etc.) nse->event_done
  * MUST be true when you call this */
-void msevent_dispatch_and_delete(mspool *nsp, msevent *nse, int notify) {
+void event_dispatch_and_delete(struct npool *nsp, struct nevent *nse, int notify) {
   assert(nsp);
   assert(nse);
 
@@ -358,7 +376,7 @@ void msevent_dispatch_and_delete(mspool *nsp, msevent *nse, int notify) {
   /* FIXME: We should be updating stats here ... */
 
   /* Now we clobber the event ... */
-  msevent_delete(nsp, nse);
+  event_delete(nsp, nse);
 }
 
 /* OK -- the idea is that we want the type included in the rightmost two bits
@@ -367,7 +385,7 @@ void msevent_dispatch_and_delete(mspool *nsp, msevent *nse, int notify) {
  * definition of a "correct" wraparound is that it goes from the highest number
  * back to one (not zero) because we don't want event numbers to ever be zero.
  * */
-nsock_event_id get_new_event_id(mspool *ms, enum nse_type type) {
+nsock_event_id get_new_event_id(struct npool *ms, enum nse_type type) {
   int type_code = (int)type;
   unsigned long serial = ms->next_event_serial++;
   unsigned long max_serial_allowed;
@@ -376,8 +394,8 @@ nsock_event_id get_new_event_id(mspool *ms, enum nse_type type) {
   assert(type < NSE_TYPE_MAX);
 
   shiftbits = sizeof(nsock_event_id) * 8 - TYPE_CODE_NUM_BITS;
-  max_serial_allowed = ( 1 << shiftbits ) - 1;
-  if (serial == max_serial_allowed ) {
+  max_serial_allowed = ((unsigned long)1 << shiftbits) - 1;
+  if (serial == max_serial_allowed) {
     /* then the next serial will be one because 0 is forbidden */
     ms->next_event_serial = 1;
   }
@@ -390,44 +408,50 @@ enum nse_type get_event_id_type(nsock_event_id event_id) {
   return (enum nse_type)((event_id & ((1 << TYPE_CODE_NUM_BITS) - 1)));
 }
 
-
-/* Create a new event structure -- must be deleted later with msevent_delete,
- * unless it returns NULL (failure).  NULL can be passed in for the msiod and
- * the userdata if not available */
-msevent *msevent_new(mspool *nsp, enum nse_type type, msiod *msiod, int timeout_msecs,
-                     nsock_ev_handler handler, void *userdata) {
-  msevent *nse;
+/* Create a new event structure -- must be deleted later with event_delete,
+ * unless it returns NULL (failure).  NULL can be passed in for the struct niod
+ * and the userdata if not available */
+struct nevent *event_new(struct npool *nsp, enum nse_type type,
+                           struct niod *iod, int timeout_msecs,
+                           nsock_ev_handler handler, void *userdata) {
+  struct nevent *nse;
+  gh_lnode_t *lnode;
 
   /* Bring us up to date for the timeout calculation. */
   gettimeofday(&nsock_tod, NULL);
 
-  if (msiod) {
-    msiod->events_pending++;
-    assert(msiod->state != NSIOD_STATE_DELETED);
+  if (iod) {
+    iod->events_pending++;
+    assert(iod->state != NSIOD_STATE_DELETED);
   }
 
   /* First we check if one is available from the free list ... */
-  nse = (msevent *)gh_list_pop(&nsp->free_events);
-  if (!nse)
-    nse = (msevent *)safe_malloc(sizeof(msevent));
-  memset(nse, 0, sizeof(msevent));
+  lnode = gh_list_pop(&nsp->free_events);
+  if (!lnode)
+    nse = (struct nevent *)safe_malloc(sizeof(*nse));
+  else
+    nse = lnode_nevent(lnode);
+
+  memset(nse, 0, sizeof(*nse));
 
   nse->id = get_new_event_id(nsp, type);
   nse->type = type;
   nse->status = NSE_STATUS_NONE;
+  gh_hnode_invalidate(&nse->expire);
 #if HAVE_OPENSSL
   nse->sslinfo.ssl_desire = SSL_ERROR_NONE;
 #endif
+
   if (type == NSE_TYPE_READ || type ==  NSE_TYPE_WRITE)
     filespace_init(&(nse->iobuf), 1024);
-#if HAVE_PCAP
 
+#if HAVE_PCAP
   if (type == NSE_TYPE_PCAP_READ) {
     mspcap *mp;
     int sz;
 
-    assert(msiod != NULL);
-    mp = (mspcap *)msiod->pcap;
+    assert(iod != NULL);
+    mp = (mspcap *)iod->pcap;
     assert(mp);
 
     sz = mp->snaplen+1 + sizeof(nsock_pcap);
@@ -440,54 +464,48 @@ msevent *msevent_new(mspool *nsp, enum nse_type type, msiod *msiod, int timeout_
     TIMEVAL_MSEC_ADD(nse->timeout, nsock_tod, timeout_msecs);
   }
 
-  nse->iod = msiod;
+  nse->iod = iod;
   nse->handler = handler;
   nse->userdata = userdata;
-  nse->time_created = nsock_tod;
 
-  if (nsp->tracelevel > 3) {
-    if (nse->iod == NULL)
-      nsock_trace(nsp, "msevent_new (IOD #NULL) (EID #%li)", nse->id);
-    else
-      nsock_trace(nsp, "msevent_new (IOD #%li) (EID #%li)", nse->iod->id, nse->id);
-  }
+  if (nse->iod == NULL)
+    nsock_log_debug("%s (IOD #NULL) (EID #%li)", __func__, nse->id);
+  else
+    nsock_log_debug("%s (IOD #%li) (EID #%li)", __func__, nse->iod->id,
+                    nse->id);
   return nse;
 }
 
-/* Free an msevent which was allocated with msevent_new, including all internal
+/* Free an struct nevent which was allocated with event_new, including all internal
  * resources.  Note -- we assume that nse->iod->events_pending (if it exists)
- * has ALREADY been decremented (done during msevent_dispatch_and_delete) -- so
- * remember to do this if you call msevent_delete() directly */
-void msevent_delete(mspool *nsp, msevent *nse) {
+ * has ALREADY been decremented (done during event_dispatch_and_delete) -- so
+ * remember to do this if you call event_delete() directly */
+void event_delete(struct npool *nsp, struct nevent *nse) {
+  if (nse->iod == NULL)
+    nsock_log_debug("%s (IOD #NULL) (EID #%li)", __func__, nse->id);
+  else
+    nsock_log_debug("%s (IOD #%li) (EID #%li)", __func__, nse->iod->id, nse->id);
 
-  if (nsp->tracelevel > 3) {
-    if (nse->iod == NULL)
-      nsock_trace(nsp, "msevent_delete (IOD #NULL) (EID #%li)", nse->id);
-    else
-      nsock_trace(nsp, "msevent_delete (IOD #%li) (EID #%li)", nse->iod->id, nse->id);
-  }
-
-  /* First free the IOBuf inside it if neccessary */
+  /* First free the IOBuf inside it if necessary */
   if (nse->type == NSE_TYPE_READ || nse->type ==  NSE_TYPE_WRITE) {
     fs_free(&nse->iobuf);
   }
   #if HAVE_PCAP
   if (nse->type == NSE_TYPE_PCAP_READ) {
     fs_free(&nse->iobuf);
-    if (nsp->tracelevel > 5)
-      nsock_trace(nsp, "PCAP removed %lu\n",nse->id);
+    nsock_log_debug_all("PCAP removed %lu", nse->id);
   }
   #endif
 
   /* Now we add the event back into the free pool */
-  gh_list_prepend(&nsp->free_events, nse);
+  nse->event_done = 1;
 }
 
 
 /* Takes an nse_type (as returned by nse_type() and returns a static string name
  * that you can use for printing, etc. */
 const char *nse_type2str(enum nse_type type) {
-  switch(type) {
+  switch (type) {
     case NSE_TYPE_CONNECT: return "CONNECT";
     case NSE_TYPE_CONNECT_SSL: return "SSL-CONNECT";
     case NSE_TYPE_READ: return "READ";
@@ -502,7 +520,7 @@ const char *nse_type2str(enum nse_type type) {
 /* Takes an nse_status (as returned by nse_status() and returns a static string
  * name that you can use for printing, etc. */
 const char *nse_status2str(enum nse_status status) {
-  switch(status) {
+  switch (status) {
     case NSE_STATUS_NONE: return "NONE";
     case NSE_STATUS_SUCCESS: return "SUCCESS";
     case NSE_STATUS_ERROR: return "ERROR";
@@ -510,8 +528,15 @@ const char *nse_status2str(enum nse_status status) {
     case NSE_STATUS_CANCELLED: return "CANCELLED";
     case NSE_STATUS_KILL: return "KILL";
     case NSE_STATUS_EOF: return "EOF";
+    case NSE_STATUS_PROXYERROR: return "PROXY ERROR";
     default:
       return "UNKNOWN!";
   }
 }
 
+int event_timedout(struct nevent *nse) {
+  if (nse->event_done)
+    return 0;
+
+  return (nse->timeout.tv_sec && !TIMEVAL_AFTER(nse->timeout, nsock_tod));
+}
