@@ -3,91 +3,134 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 with the clarifications and exceptions described  *
-# * below.  This guarantees your right to use, modify, and redistribute     *
-# * this software under certain conditions.  If you wish to embed Nmap      *
-# * technology into proprietary software, we sell alternative licenses      *
-# * (contact sales@insecure.com).  Dozens of software vendors already       *
-# * license Nmap technology such as host discovery, port scanning, OS       *
-# * detection, and version detection.                                       *
+# * The Nmap Security Scanner is (C) 1996-2019 Insecure.Com LLC ("The Nmap  *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+# * This program is free software; you may redistribute and/or modify it    *
+# * under the terms of the GNU General Public License as published by the   *
+# * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+# * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+# * right to use, modify, and redistribute this software under certain      *
+# * conditions.  If you wish to embed Nmap technology into proprietary      *
+# * software, we sell alternative licenses (contact sales@nmap.com).        *
+# * Dozens of software vendors already license Nmap technology such as      *
+# * host discovery, port scanning, OS detection, version detection, and     *
+# * the Nmap Scripting Engine.                                              *
 # *                                                                         *
-# * Note that the GPL places important restrictions on "derived works", yet *
-# * it does not provide a detailed definition of that term.  To avoid       *
-# * misunderstandings, we consider an application to constitute a           *
-# * "derivative work" for the purpose of this license if it does any of the *
-# * following:                                                              *
-# * o Integrates source code from Nmap                                      *
-# * o Reads or includes Nmap copyrighted data files, such as                *
-# *   nmap-os-db or nmap-service-probes.                                    *
-# * o Executes Nmap and parses the results (as opposed to typical shell or  *
-# *   execution-menu apps, which simply display raw Nmap output and so are  *
-# *   not derivative works.)                                                *
-# * o Integrates/includes/aggregates Nmap into a proprietary executable     *
-# *   installer, such as those produced by InstallShield.                   *
-# * o Links to a library or executes a program that does any of the above   *
+# * Note that the GPL places important restrictions on "derivative works",  *
+# * yet it does not provide a detailed definition of that term.  To avoid   *
+# * misunderstandings, we interpret that term as broadly as copyright law   *
+# * allows.  For example, we consider an application to constitute a        *
+# * derivative work for the purpose of this license if it does any of the   *
+# * following with any software or content covered by this license          *
+# * ("Covered Software"):                                                   *
 # *                                                                         *
-# * The term "Nmap" should be taken to also include any portions or derived *
-# * works of Nmap.  This list is not exclusive, but is meant to clarify our *
-# * interpretation of derived works with some common examples.  Our         *
-# * interpretation applies only to Nmap--we don't speak for other people's  *
-# * GPL works.                                                              *
+# * o Integrates source code from Covered Software.                         *
 # *                                                                         *
-# * If you have any questions about the GPL licensing restrictions on using *
-# * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
-# * we also offer alternative license to integrate Nmap into proprietary    *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates as well as helping to     *
-# * fund the continued development of Nmap technology.  Please email        *
-# * sales@insecure.com for further information.                             *
+# * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+# * or nmap-service-probes.                                                 *
 # *                                                                         *
-# * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+# * o Is designed specifically to execute Covered Software and parse the    *
+# * results (as opposed to typical shell or execution-menu apps, which will *
+# * execute anything you tell them to).                                     *
+# *                                                                         *
+# * o Includes Covered Software in a proprietary executable installer.  The *
+# * installers produced by InstallShield are an example of this.  Including *
+# * Nmap with other software in compressed or archival form does not        *
+# * trigger this provision, provided appropriate open source decompression  *
+# * or de-archiving software is widely available for no charge.  For the    *
+# * purposes of this license, an installer is considered to include Covered *
+# * Software even if it actually retrieves a copy of Covered Software from  *
+# * another source during runtime (such as by downloading it from the       *
+# * Internet).                                                              *
+# *                                                                         *
+# * o Links (statically or dynamically) to a library which does any of the  *
+# * above.                                                                  *
+# *                                                                         *
+# * o Executes a helper program, module, or script to do any of the above.  *
+# *                                                                         *
+# * This list is not exclusive, but is meant to clarify our interpretation  *
+# * of derived works with some common examples.  Other people may interpret *
+# * the plain GPL differently, so we consider this a special exception to   *
+# * the GPL that we apply to Covered Software.  Works which meet any of     *
+# * these conditions must conform to all of the terms of this license,      *
+# * particularly including the GPL Section 3 requirements of providing      *
+# * source code and allowing free redistribution of the work as a whole.    *
+# *                                                                         *
+# * As another special exception to the GPL terms, the Nmap Project grants  *
 # * permission to link the code of this program with any version of the     *
 # * OpenSSL library which is distributed under a license identical to that  *
 # * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two. You must obey the GNU GPL in all *
-# * respects for all of the code used other than OpenSSL.  If you modify    *
-# * this file, you may extend this exception to your version of the file,   *
-# * but you are not obligated to do so.                                     *
+# * linked combinations including the two.                                  *
 # *                                                                         *
-# * If you received these files with a written license agreement or         *
-# * contract stating terms other than the terms above, then that            *
-# * alternative license agreement takes precedence over these comments.     *
+# * The Nmap Project has permission to redistribute Npcap, a packet         *
+# * capturing driver and library for the Microsoft Windows platform.        *
+# * Npcap is a separate work with it's own license rather than this Nmap    *
+# * license.  Since the Npcap license does not permit redistribution        *
+# * without special permission, our Nmap Windows binary packages which      *
+# * contain Npcap may not be redistributed without special permission.      *
+# *                                                                         *
+# * Any redistribution of Covered Software, including any derived works,    *
+# * must obey and carry forward all of the terms of this license, including *
+# * obeying all GPL rules and restrictions.  For example, source code of    *
+# * the whole work must be provided and free redistribution must be         *
+# * allowed.  All GPL references to "this License", are to be treated as    *
+# * including the terms and conditions of this license text as well.        *
+# *                                                                         *
+# * Because this license imposes special exceptions to the GPL, Covered     *
+# * Work may not be combined (even as part of a larger work) with plain GPL *
+# * software.  The terms, conditions, and exceptions of this license must   *
+# * be included as well.  This license is incompatible with some other open *
+# * source licenses as well.  In some cases we can relicense portions of    *
+# * Nmap or grant special permissions to use it in other open source        *
+# * software.  Please contact fyodor@nmap.org with any such requests.       *
+# * Similarly, we don't incorporate incompatible open source software into  *
+# * Covered Software without special permission from the copyright holders. *
+# *                                                                         *
+# * If you have any questions about the licensing restrictions on using     *
+# * Nmap in other works, we are happy to help.  As mentioned above, we also *
+# * offer an alternative license to integrate Nmap into proprietary         *
+# * applications and appliances.  These contracts have been sold to dozens  *
+# * of software vendors, and generally include a perpetual license as well  *
+# * as providing support and updates.  They also fund the continued         *
+# * development of Nmap.  Please email sales@nmap.com for further           *
+# * information.                                                            *
+# *                                                                         *
+# * If you have received a written license agreement or contract for        *
+# * Covered Software stating terms other than these, you may choose to use  *
+# * and redistribute Covered Software under those terms instead of these.   *
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes (none     *
-# * have been found so far).                                                *
+# * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
 # * and add new features.  You are highly encouraged to send your changes   *
-# * to nmap-dev@insecure.org for possible incorporation into the main       *
-# * distribution.  By sending these changes to Fyodor or one of the         *
-# * Insecure.Org development mailing lists, it is assumed that you are      *
-# * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
+# * to the dev@nmap.org mailing list for possible incorporation into the    *
+# * main distribution.  By sending these changes to Fyodor or one of the    *
+# * Insecure.Org development mailing lists, or checking them into the Nmap  *
+# * source code repository, it is understood (unless you specify            *
+# * otherwise) that you are offering the Nmap Project the unlimited,        *
 # * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
-# * will always be available Open Source, but this is important because the *
-# * inability to relicense code has caused devastating problems for other   *
-# * Free Software projects (such as KDE and NASM).  We also occasionally    *
-# * relicense the code to third parties as discussed above.  If you wish to *
-# * specify special license conditions of your contributions, just say so   *
-# * when you send them.                                                     *
+# * will always be available Open Source, but this is important because     *
+# * the inability to relicense code has caused devastating problems for     *
+# * other Free Software projects (such as KDE and NASM).  We also           *
+# * occasionally relicense the code to third parties as discussed above.    *
+# * If you wish to specify special license conditions of your               *
+# * contributions, just say so when you send them.                          *
 # *                                                                         *
 # * This program is distributed in the hope that it will be useful, but     *
 # * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
-# * General Public License v2.0 for more details at                         *
-# * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
-# * included with Nmap.                                                     *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+# * license file for more details (it's in a COPYING file included with     *
+# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
 # *                                                                         *
 # ***************************************************************************/
+import sys
+
+if sys.version_info[0] != 2:
+    sys.exit("Sorry, Zenmap requires Python 2")
 
 import errno
-import sys
 import os
 import os.path
 import re
@@ -101,7 +144,8 @@ from glob import glob
 from stat import *
 
 from zenmapCore.Version import VERSION
-from zenmapCore.Name import APP_NAME, APP_DISPLAY_NAME, APP_WEB_SITE, APP_DOWNLOAD_SITE, NMAP_DISPLAY_NAME
+from zenmapCore.Name import APP_NAME, APP_DISPLAY_NAME, APP_WEB_SITE,\
+        APP_DOWNLOAD_SITE, NMAP_DISPLAY_NAME
 
 # The name of the file used to record the list of installed files, so that the
 # uninstall command can remove them.
@@ -120,6 +164,7 @@ misc_dir = os.path.join(data_dir, 'misc')
 # Where to install .desktop files.
 desktop_dir = os.path.join('share', 'applications')
 
+
 def mo_find(result, dirname, fnames):
     files = []
     for f in fnames:
@@ -130,29 +175,31 @@ def mo_find(result, dirname, fnames):
     if files:
         result.append((dirname, files))
 
-################################################################################
+###############################################################################
 # Installation variables
 
-data_files = [ (pixmaps_dir, glob(os.path.join(pixmaps_dir, '*.gif')) +
-                             glob(os.path.join(pixmaps_dir, '*.png'))),
+data_files = [
+        (pixmaps_dir, glob(os.path.join(pixmaps_dir, '*.gif')) +
+            glob(os.path.join(pixmaps_dir, '*.png'))),
 
-               (os.path.join(pixmaps_dir, "radialnet"),
-                             glob(os.path.join(pixmaps_dir, "radialnet", '*.png'))),
+        (os.path.join(pixmaps_dir, "radialnet"),
+            glob(os.path.join(pixmaps_dir, "radialnet", '*.png'))),
 
-               (config_dir, [os.path.join(config_dir, APP_NAME + '.conf')] +
-                            [os.path.join(config_dir, 'scan_profile.usp')] +
-                            [os.path.join(config_dir, APP_NAME + '_version')]),
+        (config_dir, [os.path.join(config_dir, APP_NAME + '.conf'),
+            os.path.join(config_dir, 'scan_profile.usp'),
+            os.path.join(config_dir, APP_NAME + '_version')]),
 
-               (misc_dir, glob(os.path.join(misc_dir, '*.xml'))),
+        (misc_dir, glob(os.path.join(misc_dir, '*.xml'))),
 
-               (docs_dir, [os.path.join(docs_dir, 'help.html')])]
+        (docs_dir, [os.path.join(docs_dir, 'help.html')])
+        ]
 
 # Add i18n files to data_files list
 os.path.walk(locale_dir, mo_find, data_files)
 
+
 # path_startswith and path_strip_prefix are used to deal with the installation
 # root (--root option, also known as DESTDIR).
-
 def path_startswith(path, prefix):
     """Returns True if path starts with prefix. It's a little more intelligent
     than str.startswith because it normalizes the paths to remove multiple
@@ -161,10 +208,11 @@ def path_startswith(path, prefix):
     prefix = os.path.normpath(prefix)
     return path.startswith(prefix)
 
+
 def path_strip_prefix(path, prefix):
     """Return path stripped of its directory prefix if it starts with prefix,
-    otherwise return path unmodified. This only works correctly with Unix paths;
-    for example it will not replace the drive letter on a Windows path.
+    otherwise return path unmodified. This only works correctly with Unix
+    paths; for example it will not replace the drive letter on a Windows path.
     Examples:
     >>> path_strip_prefix('/tmp/destdir/usr/bin', '/tmp/destdir')
     '/usr/bin'
@@ -187,14 +235,16 @@ def path_strip_prefix(path, prefix):
     assert os.path.isabs(path) == absolute
     return path
 
-################################################################################
+###############################################################################
 # Distutils subclasses
 
+
 class my_install(install):
     def finalize_options(self):
         # Ubuntu's python2.6-2.6.4-0ubuntu3 package changes sys.prefix in
         # install.finalize_options when sys.prefix is "/usr/local" (our
-        # default). Because we need the unchanged value later, remember it here.
+        # default). Because we need the unchanged value later, remember it
+        # here.
         self.saved_prefix = self.prefix
         install.finalize_options(self)
 
@@ -218,14 +268,17 @@ class my_install(install):
         # Recursively include all the directories in data_dir (share/zenmap).
         # This is mainly for convenience in listing locale directories.
         installed_files.append(os.path.join(self.install_data, data_dir))
-        for dirpath, dirs, files in os.walk(os.path.join(self.install_data, data_dir)):
+        for dirpath, dirs, files in os.walk(
+                os.path.join(self.install_data, data_dir)):
             for dir in dirs:
                 installed_files.append(os.path.join(dirpath, dir))
-        installed_files.append(os.path.join(self.install_scripts, "uninstall_" + APP_NAME))
+        installed_files.append(
+                os.path.join(self.install_scripts, "uninstall_" + APP_NAME))
         return installed_files
 
     def create_uninstaller(self):
-        uninstaller_filename = os.path.join(self.install_scripts, "uninstall_" + APP_NAME)
+        uninstaller_filename = os.path.join(
+                self.install_scripts, "uninstall_" + APP_NAME)
 
         uninstaller = """\
 #!/usr/bin/env python
@@ -233,26 +286,28 @@ import errno, os, os.path, sys
 
 print 'Uninstall %(name)s %(version)s'
 
-answer = raw_input('Are you sure that you want to uninstall %(name)s %(version)s? (yes/no) ')
+answer = raw_input('Are you sure that you want to uninstall '
+    '%(name)s %(version)s? (yes/no) ')
 
 if answer != 'yes' and answer != 'y':
     print 'Not uninstalling.'
     sys.exit(0)
 
-""" % {'name':APP_DISPLAY_NAME, 'version':VERSION}
+""" % {'name': APP_DISPLAY_NAME, 'version': VERSION}
 
         installed_files = []
         for output in self.get_installed_files():
             if self.root is not None:
-                # If we have a root (DESTDIR), we need to strip it off the front
-                # of paths so the uninstaller runs on the target host. The path
-                # manipulations are tricky, but made easier because the
-                # uninstaller only has to run on Unix.
+                # If we have a root (DESTDIR), we need to strip it off the
+                # front of paths so the uninstaller runs on the target host.
+                # The path manipulations are tricky, but made easier because
+                # the uninstaller only has to run on Unix.
                 if not path_startswith(output, self.root):
-                    # This should never happen (everything gets installed inside
-                    # the root), but if it does, be safe and don't delete
-                    # anything.
-                    uninstaller += "print '%s was not installed inside the root %s; skipping.'\n" % (output, self.root)
+                    # This should never happen (everything gets installed
+                    # inside the root), but if it does, be safe and don't
+                    # delete anything.
+                    uninstaller += ("print '%s was not installed inside "
+                        "the root %s; skipping.'\n" % (output, self.root))
                     continue
                 output = path_strip_prefix(output, self.root)
                 assert os.path.isabs(output)
@@ -309,26 +364,25 @@ for dir in dirs:
         # Find where the modules are installed. distutils will put them in
         # self.install_lib, but that path can contain the root (DESTDIR), so we
         # must strip it off if necessary.
-        modules = self.install_lib
+        modules_dir = self.install_lib
         if self.root is not None:
-            modules = path_strip_prefix(modules, self.root)
+            modules_dir = path_strip_prefix(modules_dir, self.root)
 
-        ufile = open(app_file_name, "r")
-        ucontent = ufile.readlines()
-        ufile.close()
+        app_file = open(app_file_name, "r")
+        lines = app_file.readlines()
+        app_file.close()
 
-        # Insert our custom import after the first non-comment line.
-        re_sys = re.compile("^#")
-        uline = 0
-        for line in ucontent:
-            if not re_sys.match(line):
+        for i in range(len(lines)):
+            if re.match(r'^INSTALL_LIB =', lines[i]):
+                lines[i] = "INSTALL_LIB = %s\n" % repr(modules_dir)
                 break
-            uline += 1
-        ucontent.insert(uline, "\nimport sys\nsys.path.append(%s)\n" % repr(modules))
+        else:
+            raise ValueError(
+                    "INSTALL_LIB replacement not found in %s" % app_file_name)
 
-        ufile = open(app_file_name, "w")
-        ufile.writelines(ucontent)
-        ufile.close()
+        app_file = open(app_file_name, "w")
+        app_file.writelines(lines)
+        app_file.close()
 
     def set_perms(self):
         re_bin = re.compile("(bin|\.sh)")
@@ -337,27 +391,28 @@ for dir in dirs:
                 continue
 
             if os.path.isdir(output):
-                os.chmod(output, S_IRWXU | \
-                                 S_IRGRP | \
-                                 S_IXGRP | \
-                                 S_IROTH | \
+                os.chmod(output, S_IRWXU |
+                                 S_IRGRP |
+                                 S_IXGRP |
+                                 S_IROTH |
                                  S_IXOTH)
             else:
-                os.chmod(output, S_IRUSR | \
-                                 S_IWUSR | \
-                                 S_IRGRP | \
+                os.chmod(output, S_IRUSR |
+                                 S_IWUSR |
+                                 S_IRGRP |
                                  S_IROTH)
 
-
     def fix_paths(self):
         """Replace some hardcoded paths to match where files were installed."""
-        interesting_paths = {"CONFIG_DIR": os.path.join(self.saved_prefix, config_dir),
-                             "DOCS_DIR": os.path.join(self.saved_prefix, docs_dir),
-                             "LOCALE_DIR": os.path.join(self.saved_prefix, locale_dir),
-                             "MISC_DIR": os.path.join(self.saved_prefix, misc_dir),
-                             "PIXMAPS_DIR": os.path.join(self.saved_prefix, pixmaps_dir),
-                             # See $(nmapdatadir) in nmap/Makefile.in.
-                             "NMAPDATADIR": os.path.join(self.saved_prefix, "share", "nmap"),}
+        interesting_paths = {
+                "CONFIG_DIR": os.path.join(self.saved_prefix, config_dir),
+                "DOCS_DIR": os.path.join(self.saved_prefix, docs_dir),
+                "LOCALE_DIR": os.path.join(self.saved_prefix, locale_dir),
+                "MISC_DIR": os.path.join(self.saved_prefix, misc_dir),
+                "PIXMAPS_DIR": os.path.join(self.saved_prefix, pixmaps_dir),
+                # See $(nmapdatadir) in nmap/Makefile.in.
+                "NMAPDATADIR": os.path.join(self.saved_prefix, "share", "nmap")
+                }
 
         # Find and read the Paths.py file.
         pcontent = ""
@@ -385,8 +440,10 @@ for dir in dirs:
         # Rewrite the zenmap.desktop and zenmap-root.desktop files to point to
         # the installed locations of the su-to-zenmap.sh script and application
         # icon.
-        su_filename = os.path.join(self.saved_prefix, data_dir, "su-to-zenmap.sh")
-        icon_filename = os.path.join(self.saved_prefix, pixmaps_dir, "zenmap.png")
+        su_filename = os.path.join(
+                self.saved_prefix, data_dir, "su-to-zenmap.sh")
+        icon_filename = os.path.join(
+                self.saved_prefix, pixmaps_dir, "zenmap.png")
 
         desktop_filename = None
         root_desktop_filename = None
@@ -410,7 +467,9 @@ for dir in dirs:
             df = open(root_desktop_filename, "r")
             dcontent = df.read()
             df.close()
-            regex = re.compile("^((?:Exec|TryExec) *= *).*su-to-zenmap.sh(.*)$", re.MULTILINE)
+            regex = re.compile(
+                    "^((?:Exec|TryExec) *= *).*su-to-zenmap.sh(.*)$",
+                    re.MULTILINE)
             dcontent = regex.sub("\\1%s\\2" % su_filename, dcontent)
             regex = re.compile("^(Icon *= *).*$", re.MULTILINE)
             dcontent = regex.sub("\\1%s" % icon_filename, dcontent)
@@ -424,7 +483,8 @@ for dir in dirs:
         doesn't strip off the installation root, if any. File names containing
         newline characters are not handled."""
         if INSTALLED_FILES_NAME == self.record:
-            distutils.log.warn("warning: installation record is overwriting --record file '%s'." % self.record)
+            distutils.log.warn("warning: installation record is overwriting "
+                "--record file '%s'." % self.record)
         f = open(INSTALLED_FILES_NAME, "w")
         try:
             for output in self.get_installed_files():
@@ -433,12 +493,14 @@ for dir in dirs:
         finally:
             f.close()
 
+
 class my_uninstall(Command):
     """A distutils command that performs uninstallation. It reads the list of
     installed files written by the install command."""
 
     command_name = "uninstall"
-    description = "uninstall installed files recorded in '%s'" % INSTALLED_FILES_NAME
+    description = "uninstall installed files recorded in '%s'" % (
+            INSTALLED_FILES_NAME)
     user_options = []
 
     def initialize_options(self):
@@ -453,7 +515,8 @@ class my_uninstall(Command):
             f = open(INSTALLED_FILES_NAME, "r")
         except IOError, e:
             if e.errno == errno.ENOENT:
-                log.error("Couldn't open the installation record '%s'. Have you installed yet?" % INSTALLED_FILES_NAME)
+                log.error("Couldn't open the installation record '%s'. "
+                        "Have you installed yet?" % INSTALLED_FILES_NAME)
                 return
         installed_files = [file.rstrip("\n") for file in f.readlines()]
         f.close()
@@ -478,7 +541,7 @@ class my_uninstall(Command):
         # Delete the directories. First reverse-sort the normalized paths by
         # length so that child directories are deleted before their parents.
         dirs = [os.path.normpath(dir) for dir in dirs]
-        dirs.sort(key = len, reverse = True)
+        dirs.sort(key=len, reverse=True)
         for dir in dirs:
             try:
                 log.info("Removing the directory '%s'." % dir)
@@ -491,24 +554,24 @@ class my_uninstall(Command):
                     log.error(str(e))
 
 # setup can be called in different ways depending on what we're doing. (For
-# example py2exe needs special handling.) These arguments are common between all
-# the operations.
+# example py2exe needs special handling.) These arguments are common between
+# all the operations.
 COMMON_SETUP_ARGS = {
     'name': APP_NAME,
-    'license': 'Nmap License (http://nmap.org/book/man-legal.html)',
+    'license': 'Nmap License (https://nmap.org/book/man-legal.html)',
     'url': APP_WEB_SITE,
     'download_url': APP_DOWNLOAD_SITE,
     'author': 'Nmap Project',
     'maintainer': 'Nmap Project',
     'description': "%s frontend and results viewer" % NMAP_DISPLAY_NAME,
-    'long_description': """\
-%s is an %s frontend \
-that is really useful for advanced users and easy to be used by newbies.""" \
-% (APP_DISPLAY_NAME, NMAP_DISPLAY_NAME),
+    'long_description': "%s is an %s frontend that is really useful"
+        "for advanced users and easy to be used by newbies." % (
+            APP_DISPLAY_NAME, NMAP_DISPLAY_NAME),
     'version': VERSION,
     'scripts': [APP_NAME],
     'packages': ['zenmapCore', 'zenmapGUI', 'zenmapGUI.higwidgets',
-                 'radialnet', 'radialnet.bestwidgets', 'radialnet.core', 'radialnet.gui', 'radialnet.util'],
+                 'radialnet', 'radialnet.bestwidgets', 'radialnet.core',
+                 'radialnet.gui', 'radialnet.util'],
     'data_files': data_files,
 }
 
@@ -523,17 +586,33 @@ if 'py2exe' in sys.argv:
     WINDOWS_SETUP_ARGS = {
         'zipfile': 'py2exe/library.zip',
         'name': APP_NAME,
-        'windows': [{"script": APP_NAME,
-                     "icon_resources": [(1, "install_scripts/windows/nmap-eye.ico")]}],
+        'windows': [{
+            "script": APP_NAME,
+            "icon_resources": [(1, "install_scripts/windows/nmap-eye.ico")]
+            }],
         # On Windows we build Ndiff here in Zenmap's setup.py so the two Python
         # programs will share a common runtime.
-        'console': [{"script": "../ndiff/ndiff", "description": "Nmap scan comparison tool"}],
+        'py_modules': ["ndiff"],
+        # override the package search path to let Ndiff be found
+        'package_dir': {
+            'zenmapCore': 'zenmapCore',
+            'zenmapGUI': 'zenmapGUI',
+            'radialnet': 'radialnet',
+            '': '../ndiff'
+            },
+        'console': [{
+            "script": "../ndiff/scripts/ndiff",
+            "description": "Nmap scan comparison tool"
+            }],
         'options': {"py2exe": {
             "compressed": 1,
             "optimize": 2,
             "packages": ["encodings"],
-            "includes": ["pango", "atk", "gobject", "gio", "pickle", "bz2", "encodings", "encodings.*", "cairo", "pangocairo"],
-            "dll_excludes": ["USP10.dll", "NSI.dll", "MSIMG32.dll", "DNSAPI.dll"]
+            "includes": ["pango", "atk", "gobject", "gio", "pickle", "bz2",
+                "encodings", "encodings.*", "cairo", "pangocairo"],
+            "dll_excludes": ["USP10.dll", "NSI.dll", "MSIMG32.dll",
+                "DNSAPI.dll"],
+            "custom_boot_script": "install_scripts/windows/boot_script.py",
             }
         }
     }
@@ -561,6 +640,9 @@ elif 'py2app' in sys.argv:
     }
 
     setup_args.update(MACOSX_SETUP_ARGS)
+elif 'vanilla' in sys.argv:
+    # Don't create uninstaller, don't fix paths. Used for bundling on OS X
+    sys.argv.remove('vanilla')
 else:
     # Default args.
     DEFAULT_SETUP_ARGS = {
