@@ -2,89 +2,126 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 with the clarifications and exceptions described  *
-# * below.  This guarantees your right to use, modify, and redistribute     *
-# * this software under certain conditions.  If you wish to embed Nmap      *
-# * technology into proprietary software, we sell alternative licenses      *
-# * (contact sales@insecure.com).  Dozens of software vendors already       *
-# * license Nmap technology such as host discovery, port scanning, OS       *
-# * detection, version detection, and the Nmap Scripting Engine.            *
+# * The Nmap Security Scanner is (C) 1996-2019 Insecure.Com LLC ("The Nmap  *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+# * This program is free software; you may redistribute and/or modify it    *
+# * under the terms of the GNU General Public License as published by the   *
+# * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+# * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+# * right to use, modify, and redistribute this software under certain      *
+# * conditions.  If you wish to embed Nmap technology into proprietary      *
+# * software, we sell alternative licenses (contact sales@nmap.com).        *
+# * Dozens of software vendors already license Nmap technology such as      *
+# * host discovery, port scanning, OS detection, version detection, and     *
+# * the Nmap Scripting Engine.                                              *
 # *                                                                         *
-# * Note that the GPL places important restrictions on "derived works", yet *
-# * it does not provide a detailed definition of that term.  To avoid       *
+# * Note that the GPL places important restrictions on "derivative works",  *
+# * yet it does not provide a detailed definition of that term.  To avoid   *
 # * misunderstandings, we interpret that term as broadly as copyright law   *
 # * allows.  For example, we consider an application to constitute a        *
-# * "derivative work" for the purpose of this license if it does any of the *
-# * following:                                                              *
-# * o Integrates source code from Nmap                                      *
-# * o Reads or includes Nmap copyrighted data files, such as                *
-# *   nmap-os-db or nmap-service-probes.                                    *
-# * o Executes Nmap and parses the results (as opposed to typical shell or  *
-# *   execution-menu apps, which simply display raw Nmap output and so are  *
-# *   not derivative works.)                                                *
-# * o Integrates/includes/aggregates Nmap into a proprietary executable     *
-# *   installer, such as those produced by InstallShield.                   *
-# * o Links to a library or executes a program that does any of the above   *
+# * derivative work for the purpose of this license if it does any of the   *
+# * following with any software or content covered by this license          *
+# * ("Covered Software"):                                                   *
 # *                                                                         *
-# * The term "Nmap" should be taken to also include any portions or derived *
-# * works of Nmap, as well as other software we distribute under this       *
-# * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
-# * but is meant to clarify our interpretation of derived works with some   *
-# * common examples.  Our interpretation applies only to Nmap--we don't     *
-# * speak for other people's GPL works.                                     *
+# * o Integrates source code from Covered Software.                         *
 # *                                                                         *
-# * If you have any questions about the GPL licensing restrictions on using *
-# * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
-# * we also offer alternative license to integrate Nmap into proprietary    *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@insecure.com for     *
-# * further information.                                                    *
+# * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+# * or nmap-service-probes.                                                 *
 # *                                                                         *
-# * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+# * o Is designed specifically to execute Covered Software and parse the    *
+# * results (as opposed to typical shell or execution-menu apps, which will *
+# * execute anything you tell them to).                                     *
+# *                                                                         *
+# * o Includes Covered Software in a proprietary executable installer.  The *
+# * installers produced by InstallShield are an example of this.  Including *
+# * Nmap with other software in compressed or archival form does not        *
+# * trigger this provision, provided appropriate open source decompression  *
+# * or de-archiving software is widely available for no charge.  For the    *
+# * purposes of this license, an installer is considered to include Covered *
+# * Software even if it actually retrieves a copy of Covered Software from  *
+# * another source during runtime (such as by downloading it from the       *
+# * Internet).                                                              *
+# *                                                                         *
+# * o Links (statically or dynamically) to a library which does any of the  *
+# * above.                                                                  *
+# *                                                                         *
+# * o Executes a helper program, module, or script to do any of the above.  *
+# *                                                                         *
+# * This list is not exclusive, but is meant to clarify our interpretation  *
+# * of derived works with some common examples.  Other people may interpret *
+# * the plain GPL differently, so we consider this a special exception to   *
+# * the GPL that we apply to Covered Software.  Works which meet any of     *
+# * these conditions must conform to all of the terms of this license,      *
+# * particularly including the GPL Section 3 requirements of providing      *
+# * source code and allowing free redistribution of the work as a whole.    *
+# *                                                                         *
+# * As another special exception to the GPL terms, the Nmap Project grants  *
 # * permission to link the code of this program with any version of the     *
 # * OpenSSL library which is distributed under a license identical to that  *
 # * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two. You must obey the GNU GPL in all *
-# * respects for all of the code used other than OpenSSL.  If you modify    *
-# * this file, you may extend this exception to your version of the file,   *
-# * but you are not obligated to do so.                                     *
+# * linked combinations including the two.                                  *
 # *                                                                         *
-# * If you received these files with a written license agreement or         *
-# * contract stating terms other than the terms above, then that            *
-# * alternative license agreement takes precedence over these comments.     *
+# * The Nmap Project has permission to redistribute Npcap, a packet         *
+# * capturing driver and library for the Microsoft Windows platform.        *
+# * Npcap is a separate work with it's own license rather than this Nmap    *
+# * license.  Since the Npcap license does not permit redistribution        *
+# * without special permission, our Nmap Windows binary packages which      *
+# * contain Npcap may not be redistributed without special permission.      *
+# *                                                                         *
+# * Any redistribution of Covered Software, including any derived works,    *
+# * must obey and carry forward all of the terms of this license, including *
+# * obeying all GPL rules and restrictions.  For example, source code of    *
+# * the whole work must be provided and free redistribution must be         *
+# * allowed.  All GPL references to "this License", are to be treated as    *
+# * including the terms and conditions of this license text as well.        *
+# *                                                                         *
+# * Because this license imposes special exceptions to the GPL, Covered     *
+# * Work may not be combined (even as part of a larger work) with plain GPL *
+# * software.  The terms, conditions, and exceptions of this license must   *
+# * be included as well.  This license is incompatible with some other open *
+# * source licenses as well.  In some cases we can relicense portions of    *
+# * Nmap or grant special permissions to use it in other open source        *
+# * software.  Please contact fyodor@nmap.org with any such requests.       *
+# * Similarly, we don't incorporate incompatible open source software into  *
+# * Covered Software without special permission from the copyright holders. *
+# *                                                                         *
+# * If you have any questions about the licensing restrictions on using     *
+# * Nmap in other works, we are happy to help.  As mentioned above, we also *
+# * offer an alternative license to integrate Nmap into proprietary         *
+# * applications and appliances.  These contracts have been sold to dozens  *
+# * of software vendors, and generally include a perpetual license as well  *
+# * as providing support and updates.  They also fund the continued         *
+# * development of Nmap.  Please email sales@nmap.com for further           *
+# * information.                                                            *
+# *                                                                         *
+# * If you have received a written license agreement or contract for        *
+# * Covered Software stating terms other than these, you may choose to use  *
+# * and redistribute Covered Software under those terms instead of these.   *
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes (none     *
-# * have been found so far).                                                *
+# * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
 # * and add new features.  You are highly encouraged to send your changes   *
-# * to nmap-dev@insecure.org for possible incorporation into the main       *
-# * distribution.  By sending these changes to Fyodor or one of the         *
+# * to the dev@nmap.org mailing list for possible incorporation into the    *
+# * main distribution.  By sending these changes to Fyodor or one of the    *
 # * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
+# * source code repository, it is understood (unless you specify            *
+# * otherwise) that you are offering the Nmap Project the unlimited,        *
+# * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+# * will always be available Open Source, but this is important because     *
+# * the inability to relicense code has caused devastating problems for     *
+# * other Free Software projects (such as KDE and NASM).  We also           *
 # * occasionally relicense the code to third parties as discussed above.    *
 # * If you wish to specify special license conditions of your               *
 # * contributions, just say so when you send them.                          *
 # *                                                                         *
 # * This program is distributed in the hope that it will be useful, but     *
 # * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
-# * General Public License v2.0 for more details at                         *
-# * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
-# * included with Nmap.                                                     *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+# * license file for more details (it's in a COPYING file included with     *
+# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
 # *                                                                         *
 # ***************************************************************************/
 
@@ -100,6 +137,12 @@ import sys
 from zenmapCore.Paths import Path
 from zenmapCore.UmitLogging import log
 
+
+class ScriptDBSyntaxError(SyntaxError):
+    """Exception raised when encountering a syntax error in the script.db"""
+    pass
+
+
 class ScriptDB (object):
     """Class responsible for parsing the script.db file, fetching script
     names and categories."""
@@ -107,31 +150,50 @@ class ScriptDB (object):
         "a": "\a", "b": "\b", "f": "\f", "n": "\n", "r": "\r",
         "t": "\t", "v": "\v", "\\": "\\", "\"": "\"", "'": "'", "0": "\0"
     }
-    def __init__(self, script_db_path = None):
+
+    def __init__(self, script_db_path=None):
         self.unget_buf = ""
 
         self.f = open(script_db_path, "r")
+        self.lineno = 1
+        self.line = ""
         try:
             self.entries_list = self.parse()
         finally:
             self.f.close()
 
+    def syntax_error(self, message):
+        e = ScriptDBSyntaxError(message)
+        e.filename = self.f.name
+        e.lineno = self.lineno
+        e.offset = len(self.line)
+        e.text = self.line
+        return e
+
     def getchar(self):
+        c = None
         if self.unget_buf:
             c = self.unget_buf[-1]
             self.unget_buf = self.unget_buf[:-1]
-            return c
         else:
-            return self.f.read(1)
+            c = self.f.read(1)
+        if c == "\n":
+            self.lineno += 1
+            self.line = ""
+        else:
+            self.line += c
+        return c
 
     def unget(self, data):
         if data:
+            self.line = self.line[:-len(data)]
             self.unget_buf += data
 
     def parse(self):
         """Parses a script.db entry and returns it as a dictionary. An entry
         looks like this:
-        Entry { filename = "afp-brute.nse", categories = { "auth", "intrusive", } }
+        Entry { filename = "afp-brute.nse", categories = \
+                { "auth", "intrusive", } }
         """
         entries = []
         while True:
@@ -165,7 +227,7 @@ class ScriptDB (object):
                     repl = None
                     c = self.getchar()
                     if not c:
-                        raise ScriptDBSyntaxError()
+                        raise self.syntax_error("Unexpected EOF")
                     if c.isdigit():
                         d1 = c
                         d2 = self.getchar()
@@ -173,7 +235,8 @@ class ScriptDB (object):
                         if d1 and d2 and d3:
                             n = int(d1 + d2 + d3)
                             if n > 255:
-                                raise ScriptDBSyntaxError()
+                                raise self.syntax_error(
+                                        "Character code >255")
                             repl = chr(n)
                         else:
                             self.unget(d3)
@@ -181,7 +244,7 @@ class ScriptDB (object):
                     if not repl:
                         repl = self.LUA_STRING_ESCAPES.get(c)
                     if not repl:
-                        raise ScriptDBSyntaxError()
+                        raise self.syntax_error("Unhandled string escape")
                     c = repl
                 string.append(c)
                 c = self.getchar()
@@ -189,13 +252,15 @@ class ScriptDB (object):
         elif c in "{},=":
             return ("delim", c)
         else:
-            raise ScriptDBSyntaxError()
+            raise self.syntax_error("Unknown token")
 
     def expect(self, tokens):
         for token in tokens:
             t = self.token()
             if t != token:
-                raise ScriptDBSyntaxError()
+                raise self.syntax_error(
+                        "Unexpected token '%s', expected '%s'" % (
+                            t[1], token[1]))
 
     def parse_entry(self):
         entry = {}
@@ -205,9 +270,10 @@ class ScriptDB (object):
         self.expect((("delim", "{"), ("ident", "filename"), ("delim", "=")))
         token = self.token()
         if not token or token[0] != "string":
-            raise ScriptDBSyntaxError()
+            raise self.syntax_error("Unexpected non-string token or EOF")
         entry["filename"] = token[1]
-        self.expect((("delim", ","), ("ident", "categories"), ("delim", "="), ("delim", "{")))
+        self.expect((("delim", ","), ("ident", "categories"),
+            ("delim", "="), ("delim", "{")))
         entry["categories"] = []
         token = self.token()
         if token and token[0] == "string":
@@ -221,17 +287,20 @@ class ScriptDB (object):
                 break
             token = self.token()
         if token != ("delim", "}"):
-            raise ScriptDBSyntaxError()
+            raise self.syntax_error(
+                    "Unexpected token '%s', expected '}'" % (token[1]))
         token = self.token()
         if token == ("delim", ","):
             token = self.token()
         if token != ("delim", "}"):
-            raise ScriptDBSyntaxError()
+            raise self.syntax_error(
+                    "Unexpected token '%s', expected '}'" % (token[1]))
         return entry
 
     def get_entries_list(self):
         return self.entries_list
 
+
 def nsedoc_tags_iter(f):
     in_doc_comment = False
     tag_name = None
@@ -265,6 +334,7 @@ def nsedoc_tags_iter(f):
                 tag_name = None
                 tag_text = None
 
+
 class ScriptMetadata (object):
     """Class responsible for parsing all the script information."""
 
@@ -274,14 +344,15 @@ class ScriptMetadata (object):
         def __init__(self, filename):
             self.filename = filename
             self.categories = []
-            self.arguments = [] # Arguments including library arguments.
+            self.arguments = []  # Arguments including library arguments.
             self.license = ""
-            self.author = ""
+            self.author = []
             self.description = ""
             self.output = ""
             self.usage = ""
 
-        url = property(lambda self: "http://nmap.org/nsedoc/scripts/" + os.path.splitext(self.filename)[0] + ".html")
+        url = property(lambda self: "https://nmap.org/nsedoc/scripts/"
+                "%s.html" % (os.path.splitext(self.filename)[0]))
 
     def __init__(self, scripts_dir, nselib_dir):
         self.scripts_dir = scripts_dir
@@ -292,21 +363,25 @@ class ScriptMetadata (object):
 
     def get_metadata(self, filename):
         entry = self.Entry(filename)
-        entry.description = self.get_string_variable(filename, "description")
-        entry.arguments = self.get_arguments(entry.filename)
-        entry.license = self.get_string_variable(filename, "license")
-        entry.author = self.get_string_variable(filename, "author")
-
-        filepath = os.path.join(self.scripts_dir, filename)
-        f = open(filepath, "r")
         try:
-            for tag_name, tag_text in nsedoc_tags_iter(f):
-                if tag_name == "output" and not entry.output:
-                    entry.output = tag_text
-                elif tag_name == "usage" and not entry.usage:
-                    entry.usage = tag_text
-        finally:
-            f.close()
+            entry.description = self.get_string_variable(filename, "description")
+            entry.arguments = self.get_arguments(entry.filename)
+            entry.license = self.get_string_variable(filename, "license")
+            entry.author = self.get_list_variable(filename, "author") or [
+                    self.get_string_variable(filename, "author")]
+
+            filepath = os.path.join(self.scripts_dir, filename)
+            f = open(filepath, "r")
+            try:
+                for tag_name, tag_text in nsedoc_tags_iter(f):
+                    if tag_name == "output" and not entry.output:
+                        entry.output = tag_text
+                    elif tag_name == "usage" and not entry.usage:
+                        entry.usage = tag_text
+            finally:
+                f.close()
+        except IOError as e:
+            entry.description = "Error getting metadata: {}".format(e)
 
         return entry
 
@@ -320,17 +395,33 @@ class ScriptMetadata (object):
         return contents
 
     def get_string_variable(self, filename, varname):
-        contents = ScriptMetadata.get_file_contents(os.path.join(self.scripts_dir, filename))
+        contents = ScriptMetadata.get_file_contents(
+            os.path.join(self.scripts_dir, filename))
         # Short string?
-        m = re.search(re.escape(varname) + r'\s*=\s*(["\'])(.*?[^\\])\1', contents)
+        m = re.search(
+            re.escape(varname) + r'\s*=\s*(["\'])(.*?[^\\])\1', contents)
         if m:
             return m.group(2)
         # Long string?
-        m = re.search(re.escape(varname) + r'\s*=\s*\[(=*)\[(.*?)\]\1\]', contents, re.S)
+        m = re.search(
+            re.escape(varname) + r'\s*=\s*\[(=*)\[(.*?)\]\1\]', contents, re.S)
         if m:
             return m.group(2)
         return None
 
+    def get_list_variable(self, filename, varname):
+        contents = ScriptMetadata.get_file_contents(
+            os.path.join(self.scripts_dir, filename))
+        m = re.search(
+            re.escape(varname) + r'\s*=\s*\{(.*?)}', contents)
+        if not m:
+            return None
+        strings = m.group(1)
+        out = []
+        for m in re.finditer(r'(["\'])(.*?[^\\])\1\s*,?', strings, re.S):
+            out.append(m.group(2))
+        return out
+
     @staticmethod
     def get_requires(filename):
         f = open(filename, "r")
@@ -365,7 +456,7 @@ class ScriptMetadata (object):
         returned as a list of (argname, description) tuples."""
         args = []
         for tag_name, tag_text in nsedoc_tags_iter(f):
-            m = re.match(r'([\w._-]+)\s*(.*)', tag_text)
+            m = re.match(r'(\S+)\s+(.*?)', tag_text, re.DOTALL)
             if (tag_name == "arg" or tag_name == "args") and m:
                 args.append((m.group(1), m.group(2)))
         return args
@@ -413,6 +504,7 @@ class ScriptMetadata (object):
             self.library_arguments[libname] = self.get_script_args(filepath)
             self.library_requires[libname] = self.get_requires(filepath)
 
+
 def get_script_entries(scripts_dir, nselib_dir):
     """Merge the information obtained so far into one single entry for
     each script and return it."""
@@ -430,7 +522,8 @@ def get_script_entries(scripts_dir, nselib_dir):
     return entries
 
 if __name__ == '__main__':
-    for entry in get_script_entries():
+    import sys
+    for entry in get_script_entries(sys.argv[1], sys.argv[2]):
         print "*" * 75
         print "Filename:", entry.filename
         print "Categories:", entry.categories
