@@ -3,89 +3,126 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 with the clarifications and exceptions described  *
-# * below.  This guarantees your right to use, modify, and redistribute     *
-# * this software under certain conditions.  If you wish to embed Nmap      *
-# * technology into proprietary software, we sell alternative licenses      *
-# * (contact sales@insecure.com).  Dozens of software vendors already       *
-# * license Nmap technology such as host discovery, port scanning, OS       *
-# * detection, version detection, and the Nmap Scripting Engine.            *
+# * The Nmap Security Scanner is (C) 1996-2019 Insecure.Com LLC ("The Nmap  *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+# * This program is free software; you may redistribute and/or modify it    *
+# * under the terms of the GNU General Public License as published by the   *
+# * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+# * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+# * right to use, modify, and redistribute this software under certain      *
+# * conditions.  If you wish to embed Nmap technology into proprietary      *
+# * software, we sell alternative licenses (contact sales@nmap.com).        *
+# * Dozens of software vendors already license Nmap technology such as      *
+# * host discovery, port scanning, OS detection, version detection, and     *
+# * the Nmap Scripting Engine.                                              *
 # *                                                                         *
-# * Note that the GPL places important restrictions on "derived works", yet *
-# * it does not provide a detailed definition of that term.  To avoid       *
+# * Note that the GPL places important restrictions on "derivative works",  *
+# * yet it does not provide a detailed definition of that term.  To avoid   *
 # * misunderstandings, we interpret that term as broadly as copyright law   *
 # * allows.  For example, we consider an application to constitute a        *
-# * "derivative work" for the purpose of this license if it does any of the *
-# * following:                                                              *
-# * o Integrates source code from Nmap                                      *
-# * o Reads or includes Nmap copyrighted data files, such as                *
-# *   nmap-os-db or nmap-service-probes.                                    *
-# * o Executes Nmap and parses the results (as opposed to typical shell or  *
-# *   execution-menu apps, which simply display raw Nmap output and so are  *
-# *   not derivative works.)                                                *
-# * o Integrates/includes/aggregates Nmap into a proprietary executable     *
-# *   installer, such as those produced by InstallShield.                   *
-# * o Links to a library or executes a program that does any of the above   *
+# * derivative work for the purpose of this license if it does any of the   *
+# * following with any software or content covered by this license          *
+# * ("Covered Software"):                                                   *
 # *                                                                         *
-# * The term "Nmap" should be taken to also include any portions or derived *
-# * works of Nmap, as well as other software we distribute under this       *
-# * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
-# * but is meant to clarify our interpretation of derived works with some   *
-# * common examples.  Our interpretation applies only to Nmap--we don't     *
-# * speak for other people's GPL works.                                     *
+# * o Integrates source code from Covered Software.                         *
 # *                                                                         *
-# * If you have any questions about the GPL licensing restrictions on using *
-# * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
-# * we also offer alternative license to integrate Nmap into proprietary    *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@insecure.com for     *
-# * further information.                                                    *
+# * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+# * or nmap-service-probes.                                                 *
 # *                                                                         *
-# * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+# * o Is designed specifically to execute Covered Software and parse the    *
+# * results (as opposed to typical shell or execution-menu apps, which will *
+# * execute anything you tell them to).                                     *
+# *                                                                         *
+# * o Includes Covered Software in a proprietary executable installer.  The *
+# * installers produced by InstallShield are an example of this.  Including *
+# * Nmap with other software in compressed or archival form does not        *
+# * trigger this provision, provided appropriate open source decompression  *
+# * or de-archiving software is widely available for no charge.  For the    *
+# * purposes of this license, an installer is considered to include Covered *
+# * Software even if it actually retrieves a copy of Covered Software from  *
+# * another source during runtime (such as by downloading it from the       *
+# * Internet).                                                              *
+# *                                                                         *
+# * o Links (statically or dynamically) to a library which does any of the  *
+# * above.                                                                  *
+# *                                                                         *
+# * o Executes a helper program, module, or script to do any of the above.  *
+# *                                                                         *
+# * This list is not exclusive, but is meant to clarify our interpretation  *
+# * of derived works with some common examples.  Other people may interpret *
+# * the plain GPL differently, so we consider this a special exception to   *
+# * the GPL that we apply to Covered Software.  Works which meet any of     *
+# * these conditions must conform to all of the terms of this license,      *
+# * particularly including the GPL Section 3 requirements of providing      *
+# * source code and allowing free redistribution of the work as a whole.    *
+# *                                                                         *
+# * As another special exception to the GPL terms, the Nmap Project grants  *
 # * permission to link the code of this program with any version of the     *
 # * OpenSSL library which is distributed under a license identical to that  *
 # * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two. You must obey the GNU GPL in all *
-# * respects for all of the code used other than OpenSSL.  If you modify    *
-# * this file, you may extend this exception to your version of the file,   *
-# * but you are not obligated to do so.                                     *
+# * linked combinations including the two.                                  *
 # *                                                                         *
-# * If you received these files with a written license agreement or         *
-# * contract stating terms other than the terms above, then that            *
-# * alternative license agreement takes precedence over these comments.     *
+# * The Nmap Project has permission to redistribute Npcap, a packet         *
+# * capturing driver and library for the Microsoft Windows platform.        *
+# * Npcap is a separate work with it's own license rather than this Nmap    *
+# * license.  Since the Npcap license does not permit redistribution        *
+# * without special permission, our Nmap Windows binary packages which      *
+# * contain Npcap may not be redistributed without special permission.      *
+# *                                                                         *
+# * Any redistribution of Covered Software, including any derived works,    *
+# * must obey and carry forward all of the terms of this license, including *
+# * obeying all GPL rules and restrictions.  For example, source code of    *
+# * the whole work must be provided and free redistribution must be         *
+# * allowed.  All GPL references to "this License", are to be treated as    *
+# * including the terms and conditions of this license text as well.        *
+# *                                                                         *
+# * Because this license imposes special exceptions to the GPL, Covered     *
+# * Work may not be combined (even as part of a larger work) with plain GPL *
+# * software.  The terms, conditions, and exceptions of this license must   *
+# * be included as well.  This license is incompatible with some other open *
+# * source licenses as well.  In some cases we can relicense portions of    *
+# * Nmap or grant special permissions to use it in other open source        *
+# * software.  Please contact fyodor@nmap.org with any such requests.       *
+# * Similarly, we don't incorporate incompatible open source software into  *
+# * Covered Software without special permission from the copyright holders. *
+# *                                                                         *
+# * If you have any questions about the licensing restrictions on using     *
+# * Nmap in other works, we are happy to help.  As mentioned above, we also *
+# * offer an alternative license to integrate Nmap into proprietary         *
+# * applications and appliances.  These contracts have been sold to dozens  *
+# * of software vendors, and generally include a perpetual license as well  *
+# * as providing support and updates.  They also fund the continued         *
+# * development of Nmap.  Please email sales@nmap.com for further           *
+# * information.                                                            *
+# *                                                                         *
+# * If you have received a written license agreement or contract for        *
+# * Covered Software stating terms other than these, you may choose to use  *
+# * and redistribute Covered Software under those terms instead of these.   *
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes (none     *
-# * have been found so far).                                                *
+# * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
 # * and add new features.  You are highly encouraged to send your changes   *
-# * to nmap-dev@insecure.org for possible incorporation into the main       *
-# * distribution.  By sending these changes to Fyodor or one of the         *
+# * to the dev@nmap.org mailing list for possible incorporation into the    *
+# * main distribution.  By sending these changes to Fyodor or one of the    *
 # * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
+# * source code repository, it is understood (unless you specify            *
+# * otherwise) that you are offering the Nmap Project the unlimited,        *
+# * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+# * will always be available Open Source, but this is important because     *
+# * the inability to relicense code has caused devastating problems for     *
+# * other Free Software projects (such as KDE and NASM).  We also           *
 # * occasionally relicense the code to third parties as discussed above.    *
 # * If you wish to specify special license conditions of your               *
 # * contributions, just say so when you send them.                          *
 # *                                                                         *
 # * This program is distributed in the hope that it will be useful, but     *
 # * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
-# * General Public License v2.0 for more details at                         *
-# * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
-# * included with Nmap.                                                     *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+# * license file for more details (it's in a COPYING file included with     *
+# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
 # *                                                                         *
 # ***************************************************************************/
 
@@ -94,6 +131,7 @@ import os
 import signal
 import sys
 import ConfigParser
+import shutil
 
 # Cause an exception if PyGTK can't open a display. Normally this just
 # produces a warning, but the lack of a display eventually causes a
@@ -102,7 +140,7 @@ import ConfigParser
 # Python 2.7 that otherwise causes an assertion failure. See
 # https://bugzilla.redhat.com/show_bug.cgi?id=620216#c10.
 import warnings
-warnings.filterwarnings("error", module = "gtk", append = "True")
+warnings.filterwarnings("error", module="gtk", append="True")
 try:
     import gtk
 except Exception:
@@ -132,14 +170,13 @@ import zenmapCore.I18N
 from zenmapCore.Paths import Path
 from zenmapCore.Name import APP_DISPLAY_NAME
 
-from zenmapGUI.MainWindow import ScanWindow
-
 from zenmapGUI.higwidgets.higdialogs import HIGAlertDialog
 
 # A global list of open scan windows. When the last one is destroyed, we call
 # gtk.main_quit.
 open_windows = []
 
+
 def _destroy_callback(window):
     open_windows.remove(window)
     if len(open_windows) == 0:
@@ -152,7 +189,9 @@ def _destroy_callback(window):
         # Cleaning up data base
         UmitDB().cleanup(SearchConfig().converted_save_time)
 
+
 def new_window():
+    from zenmapGUI.MainWindow import ScanWindow
     w = ScanWindow()
     w.connect("destroy", _destroy_callback)
     if is_maemo():
@@ -162,14 +201,23 @@ def new_window():
     open_windows.append(w)
     return w
 
-# Script found at http://www.py2exe.org/index.cgi/HowToDetermineIfRunningFromExe
+
+# Script found at
+# http://www.py2exe.org/index.cgi/HowToDetermineIfRunningFromExe
 def main_is_frozen():
-    return (hasattr(sys, "frozen") # new py2exe
-            or hasattr(sys, "importers") # old py2exe
-            or imp.is_frozen("__main__")) # tools/freeze
+    return (hasattr(sys, "frozen")  # new py2exe
+            or hasattr(sys, "importers")  # old py2exe
+            or imp.is_frozen("__main__"))  # tools/freeze
+
 
 def is_root():
-    return sys.platform == "win32" or os.getuid() == 0 or is_maemo()
+    if 'NMAP_PRIVILEGED' in os.environ:
+        return True
+    elif 'NMAP_UNPRIVILEGED' in os.environ:
+        return False
+    else:
+        return sys.platform == "win32" or os.getuid() == 0 or is_maemo()
+
 
 def install_excepthook():
     # This will catch exceptions and send them to bugzilla
@@ -182,7 +230,7 @@ def install_excepthook():
         # produces a warning, but the lack of a display eventually causes a
         # segmentation fault. See http://live.gnome.org/PyGTK/WhatsNew210.
         import warnings
-        warnings.filterwarnings("error", module = "gtk")
+        warnings.filterwarnings("error", module="gtk")
         import gtk
         warnings.resetwarnings()
 
@@ -193,10 +241,9 @@ def install_excepthook():
         if type == ImportError:
             d = HIGAlertDialog(type=gtk.MESSAGE_ERROR,
                 message_format=_("Import error"),
-                secondary_text=_("""\
-A required module was not found.
+                secondary_text=_("""A required module was not found.
 
-""" + value.message))
+""" + unicode(value)))
             d.run()
             d.destroy()
         else:
@@ -210,6 +257,7 @@ A required module was not found.
 
     sys.excepthook = excepthook
 
+
 def safe_shutdown(signum, stack):
     """Kills any active scans/tabs and shuts down the application."""
     log.debug("\n\n%s\nSAFE SHUTDOWN!\n%s\n" % ("#" * 30, "#" * 30))
@@ -220,6 +268,7 @@ def safe_shutdown(signum, stack):
 
     sys.exit(signum)
 
+
 def run():
     if os.name == "posix":
         signal.signal(signal.SIGHUP, safe_shutdown)
@@ -235,19 +284,25 @@ def run():
     try:
         # Create the ~/.zenmap directory by copying from the system-wide
         # template directory.
-        zenmapCore.Paths.create_user_config_dir(Path.user_config_dir, Path.config_dir)
+        zenmapCore.Paths.create_user_config_dir(
+                Path.user_config_dir, Path.config_dir)
     except (IOError, OSError), e:
-        error_dialog = HIGAlertDialog(message_format = _("Error creating the per-user configuration directory"),
-            secondary_text = _("""\
+        error_dialog = HIGAlertDialog(
+                message_format=_(
+                    "Error creating the per-user configuration directory"),
+                secondary_text=_("""\
 There was an error creating the directory %s or one of the files in it. \
 The directory is created by copying the contents of %s. \
-The specific error was\n\
-\n\
-%s\n\
-\n\
+The specific error was
+
+%s
+
 %s needs to create this directory to store information such as the list of \
-scan profiles. Check for access to the directory and try again.\
-""") % (repr(Path.user_config_dir), repr(Path.config_dir), repr(str(e)), APP_DISPLAY_NAME))
+scan profiles. Check for access to the directory and try again.""") % (
+                    repr(Path.user_config_dir), repr(Path.config_dir),
+                    repr(str(e)), APP_DISPLAY_NAME
+                    )
+                )
         error_dialog.run()
         error_dialog.destroy()
         sys.exit(1)
@@ -256,18 +311,39 @@ scan profiles. Check for access to the directory and try again.\
         # Read the ~/.zenmap/zenmap.conf configuration file.
         zenmapCore.UmitConf.config_parser.read(Path.user_config_file)
     except ConfigParser.ParsingError, e:
-        error_dialog = HIGAlertDialog(message_format = _("Error parsing the configuration file"),
-            secondary_text = _("""\
+        # ParsingError can leave some values as lists instead of strings. Just
+        # blow it all away if we have this problem.
+        zenmapCore.UmitConf.config_parser = zenmapCore.UmitConf.config_parser.__class__()
+        error_dialog = HIGAlertDialog(
+                message_format=_("Error parsing the configuration file"),
+                secondary_text=_("""\
 There was an error parsing the configuration file %s. \
-The specific error was\n\
-\n\
-%s\n\
-\n\
+The specific error was
+
+%s
+
 %s can continue without this file but any information in it will be ignored \
-until it is repaired.\
-""") % (Path.user_config_file, str(e), APP_DISPLAY_NAME))
+until it is repaired.""") % (Path.user_config_file, str(e), APP_DISPLAY_NAME)
+                )
         error_dialog.run()
         error_dialog.destroy()
+        global_config_path = os.path.join(Path.config_dir, APP_NAME + '.conf')
+        repair_dialog = HIGAlertDialog(
+                type=gtk.MESSAGE_QUESTION,
+                message_format=_("Restore default configuration?"),
+                secondary_text=_("""\
+To avoid further errors parsing the configuration file %s, \
+you can copy the default configuration from %s.
+
+Do this now? \
+""") % (Path.user_config_file, global_config_path),
+                )
+        repair_dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
+        repair_dialog.set_default_response(gtk.RESPONSE_CANCEL)
+        if repair_dialog.run() == gtk.RESPONSE_OK:
+            shutil.copyfile(global_config_path, Path.user_config_file)
+            log.debug(">>> Copy %s to %s." % (global_config_path, Path.user_config_file))
+        repair_dialog.destroy()
 
     # Display a "you're not root" warning if appropriate.
     if not is_root():
@@ -319,10 +395,13 @@ until it is repaired.\
     if main_is_frozen():
         gtk.gdk.threads_leave()
 
+
 class NonRootWarning (HIGAlertDialog):
     def __init__(self):
-        warning_text = _('''You are trying to run %s with a non-root user!\n
-Some %s options need root privileges to work.''') % (APP_DISPLAY_NAME, NMAP_DISPLAY_NAME)
+        warning_text = _('''You are trying to run %s with a non-root user!
+
+Some %s options need root privileges to work.''') % (
+            APP_DISPLAY_NAME, NMAP_DISPLAY_NAME)
 
         HIGAlertDialog.__init__(self, message_format=_('Non-root user'),
                                 secondary_text=warning_text)
